@@ -25,7 +25,7 @@
 struct sentry {
     struct filter  *s_filt;
     struct knote   *s_knote;
-    volatile uint32_t s_cnt;
+    atomic_uint    s_cnt;
 };
 
 static pthread_mutex_t sigtbl_mtx = PTHREAD_MUTEX_INITIALIZER;
