@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 // Do not use "#pragma once here". We use this file as top-level for header
 // pre-compilation, and GCC doesn't like that there.
@@ -10,7 +10,6 @@
 #include <hilti/rt/autogen/version.h>
 #include <hilti/rt/configuration.h>
 #include <hilti/rt/context.h>
-#include <hilti/rt/deferred-expression.h>
 #include <hilti/rt/exception.h>
 #include <hilti/rt/extension-points.h>
 #include <hilti/rt/fiber-check-stack.h>
@@ -26,6 +25,8 @@
 #include <hilti/rt/types/all.h>
 #include <hilti/rt/util.h>
 
-using namespace hilti::rt::bytes::literals; // NOLINT (google-global-names-in-headers)
+using namespace hilti::rt::bytes::literals; // NOLINT(google-global-names-in-headers)
+using namespace std::string_literals;       // NOLINT(google-global-names-in-headers)
+using namespace std::string_view_literals;  // NOLINT(google-global-names-in-headers)
 
 #endif
