@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
@@ -17,6 +17,7 @@
 #include <hilti/rt/types/bytes.h>
 #include <hilti/rt/types/map.h>
 #include <hilti/rt/types/network.h>
+#include <hilti/rt/types/null.h>
 #include <hilti/rt/types/port.h>
 #include <hilti/rt/types/reference.h>
 #include <hilti/rt/types/set.h>
@@ -89,7 +90,7 @@ public:
      * @param parent parent controlling life time of the value
      */
     Value(const void* ptr, const TypeInfo* ti, const value::Parent& parent)
-        : _ptr(ptr), _ti(ti), _parent_handle(parent.handle()) {
+        : _ptr(ptr), _ti(ti), _parent_handle(parent._handle) {
         check();
     }
 
@@ -105,6 +106,15 @@ public:
         check();
     }
 
+    /**
+     * Constructor that does not tie the value to a parent.
+     *
+     * @param ptr raw pointer to storage of the value
+     * @param ti type information describing how to interpret the pointer
+     * @param parent parent value controlling life time of this value
+     */
+    Value(const void* ptr, const TypeInfo* ti) : _ptr(ptr), _ti(ti) {}
+
     /**
      * Default constructor creating a value in invalid state.
      */
@@ -127,18 +137,27 @@ public:
     /** Returns the type information associated with the raw pointer. */
     const TypeInfo& type() const { return *_ti; }
 
+    /** Returns a user-facing string representation of the value. */
+    std::string to_string() const;
+
     /** Returns true if the instance is referring to a valid value. */
     operator bool() const { return _ptr != nullptr; }
 
 private:
     // Throws if parent has expired.
     void check() const {
+        std::weak_ptr<bool> default_;
+
+        if ( ! _parent_handle.owner_before(default_) && ! default_.owner_before(_parent_handle) )
+            // parent handle was never set
+            return;
+
         if ( _parent_handle.expired() )
             throw InvalidValue("type info value expired");
     }
 
-    const void* _ptr;
-    const TypeInfo* _ti;
+    const void* _ptr = nullptr;
+    const TypeInfo* _ti = nullptr;
     std::weak_ptr<bool> _parent_handle;
 };
 
@@ -518,7 +537,21 @@ class Function : public detail::NotImplementedType {};
 class Interval : public detail::AtomicType<hilti::rt::Interval> {};
 
 /** Auxiliary type information for type ``__library_type`. */
-class Library : public detail::ValueLessType {};
+class Library : public detail::AtomicType<hilti::rt::TypeInfo*> {
+public:
+    /**
+     * Constructor.
+     *
+     * @param cxx_name C++-side name of the type
+     */
+    Library(std::string cxx_name) : cxx_name(std::move(cxx_name)) {}
+
+    /** Returns the C++-side name of the type. */
+    const auto& cxxName() const { return cxx_name; }
+
+private:
+    const std::string cxx_name;
+};
 
 class Map;
 
@@ -764,6 +797,9 @@ private:
 /** Auxiliary type information for type ``net`. */
 class Network : public detail::AtomicType<hilti::rt::Network> {};
 
+/** Auxiliary type information for type ``null`. */
+class Null : public detail::ValueLessType {};
+
 /** Auxiliary type information for type ``optional<T>`. */
 class Optional : public detail::DereferenceableType {
 public:
@@ -1207,7 +1243,7 @@ namespace hilti::rt {
  */
 struct TypeInfo {
     std::optional<const char*> id; /**< Spicy-side ID associated with the type, if any. */
-    const char* display;           /**< String rendering of the type. */
+    const char* display = nullptr; /**< String rendering of the type. */
 
     enum Tag {
         Undefined,
@@ -1226,6 +1262,7 @@ struct TypeInfo {
         Map,
         MapIterator,
         Network,
+        Null,
         Optional,
         Port,
         Real,
@@ -1260,6 +1297,10 @@ struct TypeInfo {
     // Actual storage for the held type.
     std::unique_ptr<const char, void (*)(const char*)> _storage = {nullptr, [](const char*) {}};
 
+    // Callback rendering a value of the type into a string. User-code should
+    // use `Value::to_string()` instead of calling this directly.
+    std::string (*const _to_string)(const void* const) = nullptr;
+
     Tag tag = Tag::Undefined; ///< Tag indicating which field of below union is set.
     union {
         type_info::Address* address;
@@ -1277,6 +1318,7 @@ struct TypeInfo {
         type_info::Map* map;
         type_info::MapIterator* map_iterator;
         type_info::Network* network;
+        type_info::Null* null;
         type_info::Optional* optional;
         type_info::Port* port;
         type_info::Real* real;
@@ -1311,10 +1353,13 @@ struct TypeInfo {
     TypeInfo() = default;
 
     template<typename Type>
-    TypeInfo(std::optional<const char*> _id, const char* _display, Type* value)
-        : id(_id), display(_display), _storage(reinterpret_cast<const char*>(value), [](const char* p) {
-              delete reinterpret_cast<const Type*>(p);
-          }) {
+    TypeInfo(std::optional<const char*> _id, const char* _display, std::string (*const to_string)(const void*),
+             Type* value)
+        : id(_id),
+          display(_display),
+          _storage(reinterpret_cast<const char*>(value),
+                   [](const char* p) { delete reinterpret_cast<const Type*>(p); }),
+          _to_string(to_string) {
         if constexpr ( std::is_same_v<Type, type_info::Address> ) {
             tag = Address;
             address = value;
@@ -1375,6 +1420,10 @@ struct TypeInfo {
             tag = Network;
             network = value;
         }
+        else if constexpr ( std::is_same_v<Type, type_info::Null> ) {
+            tag = Null;
+            null = value;
+        }
         else if constexpr ( std::is_same_v<Type, type_info::Optional> ) {
             tag = Optional;
             optional = value;
@@ -1497,8 +1546,26 @@ struct TypeInfo {
     }
 };
 
+inline std::ostream& operator<<(std::ostream& out, const TypeInfo& t) { return out << t.display; }
+inline std::ostream& operator<<(std::ostream& out, const TypeInfo* t) { return out << t->display; }
+
+namespace detail::adl {
+inline std::string to_string(const hilti::rt::TypeInfo& ti, adl::tag /*unused*/) { return ti.display; }
+inline std::string to_string(const hilti::rt::TypeInfo* ti, adl::tag /*unused*/) { return ti->display; }
+} // namespace detail::adl
+
 namespace type_info {
 
+inline std::string Value::to_string() const {
+    if ( ! _ti->_to_string )
+        // This should only not happen outside of debugging and testing, but
+        // make sure we catch it. Not using `assert()` to keep this enabled in
+        // production code
+        throw AssertionFailure("type-info has no to_string() callback");
+
+    return _ti->_to_string(pointer());
+}
+
 namespace value {
 /**
  * Retrieves the auxiliary type information for a value, casted it to the
@@ -1573,6 +1640,10 @@ const Type* auxType(const type_info::Value& v) {
         assert(type.tag == TypeInfo::Network);
         return type.network;
     }
+    else if constexpr ( std::is_same_v<Type, type_info::Null> ) {
+        assert(type.tag == TypeInfo::Null);
+        return type.null;
+    }
     else if constexpr ( std::is_same_v<Type, type_info::Optional> ) {
         assert(type.tag == TypeInfo::Optional);
         return type.optional;
@@ -1697,34 +1768,33 @@ const Type* auxType(const type_info::Value& v) {
 
 
 // Forward declare static built-in type information objects.
-extern TypeInfo address;
-extern TypeInfo any;
-extern TypeInfo bool_;
-extern TypeInfo bytes_iterator;
-extern TypeInfo bytes;
-
-extern TypeInfo error;
-extern TypeInfo int16;
-extern TypeInfo int32;
-extern TypeInfo int64;
-extern TypeInfo int8;
-extern TypeInfo interval;
-extern TypeInfo library;
-extern TypeInfo network;
-extern TypeInfo port;
-extern TypeInfo real;
-extern TypeInfo regexp;
-extern TypeInfo stream_iterator;
-extern TypeInfo stream_view;
-extern TypeInfo stream;
-extern TypeInfo string;
-
-extern TypeInfo time;
-extern TypeInfo uint16;
-extern TypeInfo uint32;
-extern TypeInfo uint64;
-extern TypeInfo uint8;
-extern TypeInfo void_;
+extern const TypeInfo address;
+extern const TypeInfo any;
+extern const TypeInfo bool_;
+extern const TypeInfo bytes_iterator;
+extern const TypeInfo bytes;
+extern const TypeInfo error;
+extern const TypeInfo int16;
+extern const TypeInfo int32;
+extern const TypeInfo int64;
+extern const TypeInfo int8;
+extern const TypeInfo interval;
+extern const TypeInfo library;
+extern const TypeInfo network;
+extern const TypeInfo null;
+extern const TypeInfo port;
+extern const TypeInfo real;
+extern const TypeInfo regexp;
+extern const TypeInfo stream_iterator;
+extern const TypeInfo stream_view;
+extern const TypeInfo stream;
+extern const TypeInfo string;
+extern const TypeInfo time;
+extern const TypeInfo uint16;
+extern const TypeInfo uint32;
+extern const TypeInfo uint64;
+extern const TypeInfo uint8;
+extern const TypeInfo void_;
 
 } // namespace type_info
 
