@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <cinttypes>
 #include <memory>
@@ -14,7 +14,7 @@ using namespace hilti::rt::detail;
 namespace hilti::rt::context::detail {
 
 // Not part of global state, it's per thread.
-HILTI_THREAD_LOCAL Context* __current = nullptr;
+static HILTI_THREAD_LOCAL Context* __current = nullptr;
 
 Context*& current() { return __current; }
 
