@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <unistd.h>
 
@@ -42,12 +42,6 @@ ostream& operator<<(ostream& stream, const pair<U, V>& p) {
     return stream << "(" << p.first << ", " << p.second << ")";
 }
 
-template<typename... Ts>
-ostream& operator<<(ostream& stream, const tuple<Ts...>& xs) {
-    stream << "(";
-    tuple_for_each(xs, [&](auto&& x) { stream << x << ", "; });
-    return stream << ")";
-}
 } // namespace std
 
 TEST_SUITE_BEGIN("util");
@@ -148,7 +142,7 @@ TEST_CASE("createTemporaryFile") {
         SUBCASE("custom prefix") {
             auto prefix = "1234567890";
             tmp = createTemporaryFile(prefix).valueOrThrow();
-            CHECK(startsWith(tmp.filename(), prefix));
+            CHECK(startsWith(tmp.filename().string(), prefix));
         }
 
         CAPTURE(tmp);
@@ -166,6 +160,21 @@ TEST_CASE("createTemporaryFile") {
     }
 }
 
+TEST_CASE("endsWith") {
+    CHECK(endsWith("", ""));
+    CHECK_FALSE(endsWith("", "a"));
+    CHECK(endsWith("abc", "c"));
+    CHECK_FALSE(endsWith("abc", "a1"));
+    CHECK(endsWith("abc", "bc"));
+    CHECK(endsWith("abc", "abc"));
+
+    const auto null = std::string(1U, '\0');
+
+    CHECK(endsWith("abc" + null, null));
+    CHECK(endsWith("abc" + null, "c" + null));
+    CHECK_FALSE(endsWith("abc" + null, "abc"));
+}
+
 TEST_CASE("enumerate") {
     auto input = std::vector<char>({'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'});
     for ( auto x : enumerate(input) ) {
@@ -332,19 +341,6 @@ TEST_CASE("join") {
     CHECK_EQ(join(str_list{null, null}, null), null + null + null);
 }
 
-TEST_CASE("join_tuple") {
-    CHECK_EQ(join_tuple(std::make_tuple()), "");
-    CHECK_EQ(join_tuple(std::make_tuple(integer::safe<uint8_t>(1), std::string("a"))), "1, \"a\"");
-    CHECK_EQ(join_tuple(std::make_tuple(integer::safe<uint8_t>(1), std::string(1U, '\0'))), "1, \"\\0\"");
-}
-
-TEST_CASE("join_tuple_for_print") {
-    CHECK_EQ(join_tuple_for_print(std::make_tuple()), "");
-    CHECK_EQ(join_tuple_for_print(std::make_tuple(integer::safe<uint8_t>(1), std::string("a"))), "1, a");
-    const auto null = std::string(1U, '\0');
-    CHECK_EQ(join_tuple_for_print(std::make_tuple(integer::safe<uint8_t>(1), null)), "1, " + null);
-}
-
 TEST_CASE("ltrim") {
     CHECK_EQ(ltrim("", ""), "");
     CHECK_EQ(ltrim("", "abc"), "");
@@ -357,6 +353,13 @@ TEST_CASE("ltrim") {
 }
 
 TEST_CASE("map_tuple") {
+    auto u0 = static_cast<uint64_t>(0);
+    auto u1 = static_cast<uint64_t>(1);
+    auto u2 = static_cast<uint64_t>(2);
+    auto i0 = static_cast<int64_t>(0);
+    auto i1 = static_cast<int64_t>(1);
+    auto i2 = static_cast<int64_t>(2);
+
     CHECK_EQ(map_tuple(std::make_tuple(), []() {}), std::make_tuple());
     CHECK_EQ(map_tuple(std::make_tuple(), []() { return 0; }), std::make_tuple());
     CHECK_EQ(map_tuple(std::make_tuple(), [](auto&&) {}), std::make_tuple());
@@ -364,20 +367,20 @@ TEST_CASE("map_tuple") {
     CHECK_EQ(map_tuple(std::make_tuple(), [](auto&) {}), std::make_tuple());
     CHECK_EQ(map_tuple(std::make_tuple(), [](auto&&) { return 0; }), std::make_tuple());
     CHECK_EQ(map_tuple(std::make_tuple(), [](auto&& x) { return decltype(x){}; }), std::make_tuple());
-    CHECK_EQ(map_tuple(std::make_tuple(1U, 1L, std::string("a")), [](auto&& x) { return decltype(x){}; }),
-             std::make_tuple(0U, 0L, std::string()));
-    CHECK_EQ(map_tuple(std::make_tuple(1U, 1L, std::string("a")), [](auto x) { return std::move(x); }),
-             std::make_tuple(1U, 1L, std::string("a")));
+    CHECK_EQ(map_tuple(std::make_tuple(u1, i1, std::string("a")), [](auto&& x) { return decltype(x){}; }),
+             std::make_tuple(u0, i0, std::string()));
+    CHECK_EQ(map_tuple(std::make_tuple(u1, i1, std::string("a")), [](auto x) { return std::move(x); }),
+             std::make_tuple(u1, i1, std::string("a")));
 
-    auto input = std::make_tuple(1U, 1L, std::string("a"));
+    auto input = std::make_tuple(u1, i1, std::string("a"));
     CHECK_EQ(map_tuple(input,
                        [](auto& x) {
                            auto y = x;
                            x += x;
                            return y;
                        }),
-             std::make_tuple(1U, 1L, std::string("a")));
-    CHECK_EQ(input, std::make_tuple(2U, 2L, std::string("aa")));
+             std::make_tuple(u1, i1, std::string("a")));
+    CHECK_EQ(input, std::make_tuple(u2, i2, std::string("aa")));
 }
 
 TEST_CASE("memory_statistics") {
