@@ -1,4 +1,4 @@
-// Copyright (c) 2021-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
@@ -24,10 +24,10 @@ public:
     /** Returns the AST context in use for creating nodes. */
     ASTContext* context() const { return _context; }
 
-    auto attribute(hilti::Attribute::Kind kind, Expression* v, const Meta& m = Meta()) {
+    auto attribute(hilti::attribute::Kind kind, Expression* v, const Meta& m = Meta()) {
         return hilti::Attribute::create(context(), kind, v, m);
     }
-    auto attribute(hilti::Attribute::Kind kind, const Meta& m = Meta()) {
+    auto attribute(hilti::attribute::Kind kind, const Meta& m = Meta()) {
         return hilti::Attribute::create(context(), kind, m);
     }
     auto attributeSet(Attributes attrs = {}, Meta m = Meta()) {
@@ -100,7 +100,7 @@ public:
     }
     auto ctorPort(hilti::rt::Port v, const Meta& meta = {}) { return hilti::ctor::Port::create(context(), v, meta); }
     auto ctorReal(double v, const Meta& meta = {}) { return hilti::ctor::Real::create(context(), v, meta); }
-    auto ctorRegExp(std::vector<std::string> v, AttributeSet* attrs = nullptr, const Meta& meta = {}) {
+    auto ctorRegExp(hilti::ctor::regexp::Patterns v, AttributeSet* attrs = nullptr, const Meta& meta = {}) {
         return hilti::ctor::RegExp::create(context(), std::move(v), attrs, meta);
     }
     auto ctorResult(Expression* expr, const Meta& meta = {}) {
@@ -290,12 +290,6 @@ public:
     auto expressionCtor(Ctor* ctor, Meta meta = {}) {
         return hilti::expression::Ctor::create(context(), ctor, std::move(meta));
     }
-    auto expressionDeferred(Expression* expr, bool catch_exception, const Meta& meta = {}) {
-        return hilti::expression::Deferred::create(context(), expr, catch_exception, meta);
-    }
-    auto expressionDeferred(Expression* expr, const Meta& meta = {}) {
-        return hilti::expression::Deferred::create(context(), expr, meta);
-    }
     auto expressionGrouping(Expression* expr, Meta meta = {}) {
         return hilti::expression::Grouping::create(context(), expr, std::move(meta));
     }
@@ -497,8 +491,11 @@ public:
         return hilti::type::Function::create(context(), _, m);
     }
     auto typeInterval(Meta meta = {}) { return hilti::type::Interval::create(context(), std::move(meta)); }
-    auto typeLibrary(const std::string& cxx_name, Meta meta = {}) {
-        return hilti::type::Library::create(context(), cxx_name, std::move(meta));
+    auto typeLibrary(std::string cxx_name, Meta meta = {}) {
+        return hilti::type::Library::create(context(), Constness::Mutable, std::move(cxx_name), std::move(meta));
+    }
+    auto typeLibrary(Constness const_, std::string cxx_name, Meta meta = {}) {
+        return hilti::type::Library::create(context(), const_, std::move(cxx_name), std::move(meta));
     }
     auto typeList(QualifiedType* t, const Meta& meta = {}) { return hilti::type::List::create(context(), t, meta); }
     auto typeList(type::Wildcard _, const Meta& m = Meta()) { return hilti::type::List::create(context(), _, m); }
