@@ -1,11 +1,8 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
-#include <memory>
-#include <string>
 #include <utility>
-#include <vector>
 
 #include <hilti/ast/ctor.h>
 #include <hilti/ast/types/struct.h>
@@ -24,7 +21,7 @@ public:
 
     node::Properties properties() const override {
         auto p = node::Properties{{"id", _id}};
-        return Node::properties() + p;
+        return Node::properties() + std::move(p);
     }
 
     static auto create(ASTContext* ctx, ID id, Expression* expr, Meta meta = {}) {
@@ -69,15 +66,15 @@ public:
     /** Implements the node interface. */
     node::Properties properties() const override {
         auto p = node::Properties{};
-        return Ctor::properties() + node::WithUniqueID::properties() + p;
+        return Ctor::properties() + node::WithUniqueID::properties() + std::move(p);
     }
 
-    static auto create(ASTContext* ctx, struct_::Fields fields, QualifiedType* t, Meta meta = {}) {
-        return ctx->make<Struct>(ctx, node::flatten(t, std::move(fields)), std::move(meta));
+    static auto create(ASTContext* ctx, const struct_::Fields& fields, QualifiedType* t, Meta meta = {}) {
+        return ctx->make<Struct>(ctx, node::flatten(t, fields), std::move(meta));
     }
 
-    static auto create(ASTContext* ctx, struct_::Fields fields, const Meta& meta = {}) {
-        return ctx->make<Struct>(ctx, node::flatten(QualifiedType::createAuto(ctx, meta), std::move(fields)), meta);
+    static auto create(ASTContext* ctx, const struct_::Fields& fields, const Meta& meta = {}) {
+        return ctx->make<Struct>(ctx, node::flatten(QualifiedType::createAuto(ctx, meta), fields), meta);
     }
 
 protected:
