@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
@@ -48,17 +48,17 @@ public:
     }
 
     hilti::Expression* default_() const {
-        if ( auto a = attributes()->find(hilti::Attribute::Kind::Default) )
+        if ( auto a = attributes()->find(hilti::attribute::kind::Default) )
             return *a->valueAsExpression();
         else
             return {};
     }
 
-    auto isAnonymous() const { return attributes()->find(hilti::Attribute::Kind::Anonymous) != nullptr; }
-    auto isInternal() const { return attributes()->find(hilti::Attribute::Kind::Internal) != nullptr; }
-    auto isOptional() const { return attributes()->find(hilti::Attribute::Kind::Optional) != nullptr; }
-    auto isStatic() const { return attributes()->find(hilti::Attribute::Kind::Static) != nullptr; }
-    auto isNoEmit() const { return attributes()->find(hilti::Attribute::Kind::NoEmit) != nullptr; }
+    auto isAnonymous() const { return attributes()->find(hilti::attribute::kind::Anonymous) != nullptr; }
+    auto isInternal() const { return attributes()->find(hilti::attribute::kind::Internal) != nullptr; }
+    auto isOptional() const { return attributes()->find(hilti::attribute::kind::Optional) != nullptr; }
+    auto isStatic() const { return attributes()->find(hilti::attribute::kind::Static) != nullptr; }
+    auto isNoEmit() const { return attributes()->find(hilti::attribute::kind::NoEmit) != nullptr; }
 
     auto linkedTypeIndex() const { return _linked_type_index; }
 
@@ -78,7 +78,7 @@ public:
         if ( ! attrs )
             attrs = AttributeSet::create(ctx);
 
-        if ( attrs->has(hilti::Attribute::Kind::Static) )
+        if ( attrs->has(hilti::attribute::kind::Static) )
             // make it assignable
             type = type->recreateAsLhs(ctx);
 
