@@ -1,11 +1,10 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
 #include <memory>
 #include <set>
 #include <string>
-#include <unordered_map>
 #include <utility>
 #include <vector>
 
@@ -101,14 +100,14 @@ public:
                                   {"scope", _scope_path},
                                   {"dependencies", util::join(_dependencies, ", ")},
                                   {"skip-implementation", _skip_implementation}};
-        return hilti::Declaration::properties() + p;
+        return hilti::Declaration::properties() + std::move(p);
     }
 
     std::string_view branchTag() const final { return _uid.process_extension.native(); }
 
     static auto create(ASTContext* ctx, const declaration::module::UID& uid, const ID& scope, const Declarations& decls,
-                       Statements stmts, Meta meta = {}) {
-        Nodes nodes = {statement::Block::create(ctx, std::move(stmts), meta)};
+                       const Statements& stmts, Meta meta = {}) {
+        Nodes nodes = {statement::Block::create(ctx, stmts, meta)};
         for ( auto d : decls )
             nodes.push_back(d);
 
