@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
@@ -13,7 +13,7 @@ namespace hilti::type {
 /** AST node for type referenced by Name. */
 class Name : public UnqualifiedType {
 public:
-    auto id() const { return _id; }
+    const auto& id() const { return _id; }
     bool isBuiltIn() const { return _builtin; }
 
     // resolves recursively
@@ -51,7 +51,7 @@ public:
     node::Properties properties() const final {
         auto p =
             node::Properties{{"id", _id}, {"builtin", _builtin}, {"resolved-type", to_string(_resolved_type_index)}};
-        return UnqualifiedType::properties() + p;
+        return UnqualifiedType::properties() + std::move(p);
     }
 
     static auto create(ASTContext* ctx, const ID& id, Meta meta = {}) {
