@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
@@ -32,7 +32,7 @@ public:
      */
     explicit DebugStream(const std::string& name);
     bool operator<(const DebugStream& other) const { return _id < other._id; }
-    auto name() const { return _name; }
+    const auto& name() const { return _name; }
 
     /** Returns the names of all available debug streams. */
     static std::vector<std::string> all();
@@ -65,7 +65,7 @@ constexpr util::enum_::Value<Level> Levels[] = {
 constexpr auto to_string(Level m) { return util::enum_::to_string(m, detail::Levels); }
 
 namespace level {
-constexpr auto from_string(const std::string_view& s) { return util::enum_::from_string<Level>(s, detail::Levels); }
+constexpr auto from_string(std::string_view s) { return util::enum_::from_string<Level>(s, detail::Levels); }
 } // namespace level
 
 /** Ostream-variant that forwards output to the central logger. */
