@@ -1,4 +1,4 @@
-// Copyright (c) 2021-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <hilti/ast/builder/builder.h>
 
@@ -116,6 +116,9 @@ public:
                          .param0 = {.name = "charset",
                                     .type = {parameter::Kind::In, builder->typeName("hilti::Charset")},
                                     .default_ = builder->expressionName("hilti::Charset::UTF8")},
+                         .param1 = {.name = "errors",
+                                    .type = {parameter::Kind::In, builder->typeName("hilti::DecodeErrorStrategy")},
+                                    .default_ = builder->expressionName("hilti::DecodeErrorStrategy::REPLACE")},
                          .result = {Constness::Const, builder->typeBytes()},
                          .ns = "string",
                          .doc =
@@ -212,6 +215,28 @@ public:
 };
 HILTI_OPERATOR_IMPLEMENTATION(StartsWith);
 
+class EndsWith : public BuiltInMemberCall {
+public:
+    Signature signature(Builder* builder) const final {
+        return Signature{
+            .kind = Kind::MemberCall,
+            .self = {parameter::Kind::In, builder->typeString()},
+            .member = "ends_with",
+            .param0 =
+                {
+                    .name = "suffix",
+                    .type = {parameter::Kind::In, builder->typeString()},
+                },
+            .result = {Constness::Const, builder->typeBool()},
+            .ns = "string",
+            .doc = "Returns true if the string value ends with *suffix*.",
+        };
+    }
+
+    HILTI_OPERATOR(hilti, string::EndsWith);
+};
+HILTI_OPERATOR_IMPLEMENTATION(EndsWith);
+
 class LowerCase : public BuiltInMemberCall {
 public:
     Signature signature(Builder* builder) const final {
