@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <utility>
 
@@ -84,7 +84,7 @@ std::vector<Scope::Referee> Scope::_findID(const Scope* scope, const ID& id, boo
 
         std::string nt;
         std::tie(h, nt) = util::rsplit1(h, "::");
-        t = (! t.empty() && ! nt.empty() && t != "$ $" ? util::fmt("%s::%s", nt, t) : nt);
+        t = (! t.empty() && ! nt.empty() && t != "$ $" ? util::fmt("%s::%s", nt, t) : std::move(nt));
     }
 }
 
