@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <hilti/ast/forward.h>
 #include <hilti/ast/scope-lookup.h>
@@ -39,7 +39,7 @@ bool UnqualifiedType::isOnHeap() const {
 
 ID UnqualifiedType::cxxID() const {
     if ( auto decl = typeDeclaration() ) {
-        if ( auto a = decl->attributes()->find(hilti::Attribute::Kind::Cxxname) )
+        if ( auto a = decl->attributes()->find(hilti::attribute::kind::Cxxname) )
             return ID{*a->valueAsString()};
     }
 
@@ -51,7 +51,7 @@ hilti::node::Properties UnqualifiedType::properties() const {
                                {"type", to_string(_type_index)},
                                {"declaration", to_string(_declaration_index)},
                                {"wildcard", _is_wildcard}}};
-    return Node::properties() + p;
+    return Node::properties() + std::move(p);
 }
 
 std::string UnqualifiedType::_dump() const {
@@ -91,7 +91,8 @@ type::Name* QualifiedType::alias() const { return _type()->tryAs<type::Name>();
 hilti::node::Properties QualifiedType::properties() const {
     auto side = (_side == Side::LHS ? "lhs" : "rhs");
     auto constness = (_constness == Constness::Const ? "true" : "false");
-    return {{"const", constness}, {"side", side}};
+    auto external = (_external ? "true" : "false");
+    return {{"const", constness}, {"side", side}, {"extern", external}};
 }
 
 std::string QualifiedType::_dump() const {
