@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <hilti/ast/ast-context.h>
 #include <hilti/ast/builder/builder.h>
@@ -93,7 +93,7 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
         body.addStatement("return ::hilti::rt::detail::moduleGlobals<__globals_t>(__globals_index)");
 
         auto body_decl = cxx::declaration::Function(cxx::declaration::Function::Free, "auto", {ns, "__globals"}, {},
-                                                    "static", cxx::declaration::Function::Inline(), body);
+                                                    "static", cxx::declaration::Function::Inline(), std::move(body));
 
         unit->add(body_decl);
     }
@@ -136,7 +136,7 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
 
         cg->popCxxBlock();
 
-        auto body_decl = cxx::declaration::Function(cxx::declaration::Function::Free, "void", id,
+        auto body_decl = cxx::declaration::Function(cxx::declaration::Function::Free, "void", std::move(id),
                                                     {{"ctx", "::hilti::rt::Context*"}}, "extern", std::move(body));
         unit->add(body_decl);
     }
@@ -160,7 +160,7 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
             body.addStatement(fmt("::%s.reset();", g.id));
         }
 
-        auto body_decl = cxx::declaration::Function(cxx::declaration::Function::Free, "void", id,
+        auto body_decl = cxx::declaration::Function(cxx::declaration::Function::Free, "void", std::move(id),
                                                     {{"ctx", "::hilti::rt::Context*"}}, "extern", std::move(body));
         unit->add(body_decl);
     }
@@ -237,7 +237,7 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
 
         // Record the global for now, final declarations will be added later
         // once the visitor knows all globals.
-        globals.push_back(x);
+        globals.push_back(std::move(x));
     }
 
     void operator()(declaration::Constant* n) final {
@@ -255,8 +255,12 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
         assert(n->typeID());
 
         auto t = cg->compile(n->type(), codegen::TypeUsage::Storage);
-        if ( auto dt = cg->typeDeclaration(n->type()) )
+        if ( auto dt = cg->typeDeclaration(n->type()) ) {
+            if ( n->linkage() == declaration::Linkage::Public )
+                dt->public_ = true;
+
             unit->add(*dt);
+        }
 
         if ( include_implementation )
             cg->addTypeInfoDefinition(n->type());
@@ -265,8 +269,8 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
     void operator()(declaration::Function* n) final {
         // TODO(robin): This method needs a refactoring.
 
-        if ( n->function()->attributes()->find(hilti::Attribute::Kind::Cxxname) &&
-             n->function()->attributes()->find(hilti::Attribute::Kind::HavePrototype) )
+        if ( n->function()->attributes()->find(hilti::attribute::kind::Cxxname) &&
+             n->function()->attributes()->find(hilti::attribute::kind::HavePrototype) )
             return;
 
         const auto& f = n->function();
@@ -283,7 +287,7 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
 
         auto id_class = n->id().sub(-2);
         auto id_local = n->id().sub(-1);
-        auto id_struct_type = (id_module != current_module ? ID(id_module, id_class) : id_class);
+        const auto& id_struct_type = (id_module != current_module ? ID(id_module, id_class) : id_class);
 
         cxx::ID cid;
         if ( ! is_hook ) {
@@ -296,7 +300,7 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
 
         auto d = cg->compile(n, ft, linkage, f->callingConvention(), f->attributes(), cid);
 
-        if ( auto a = n->function()->attributes()->find(hilti::Attribute::Kind::Cxxname) ) {
+        if ( auto a = n->function()->attributes()->find(hilti::attribute::kind::Cxxname) ) {
             // Just add the prototype. Make sure to skip any custom namespacing.
             const auto& value = a->valueAsString();
             if ( ! value ) {
@@ -312,7 +316,7 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
 
         int64_t priority = 0;
         if ( is_hook && f->attributes() ) {
-            if ( auto x = f->attributes()->find(hilti::Attribute::Kind::Priority) ) {
+            if ( auto x = f->attributes()->find(hilti::attribute::kind::Priority) ) {
                 if ( auto i = x->valueAsInteger() )
                     priority = *i;
                 else
@@ -335,7 +339,7 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
 
             // Adapt the function we generate.
             d.linkage = "extern";
-            d.id = id_hook_impl;
+            d.id = std::move(id_hook_impl);
             d.ftype = cxx::declaration::Function::Free;
 
             // TODO(robin): This should compile the struct type, not hardcode
@@ -441,9 +445,9 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
 
             if ( include_implementation ) {
                 // Tell linker about our implementation.
-                auto hook_join = cxx::linker::Join{.id = id_hook_stub,
+                auto hook_join = cxx::linker::Join{.id = std::move(id_hook_stub),
                                                    .callee = d,
-                                                   .aux_types = aux_types,
+                                                   .aux_types = std::move(aux_types),
                                                    .priority = priority,
                                                    .declare_only = (! f->body())};
 
@@ -538,8 +542,9 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
                 cb.addReturn("::hilti::rt::Nothing()");
             }
 
-            body.addLambda("cb", "[args_on_heap](::hilti::rt::resumable::Handle* r) -> ::hilti::rt::any",
-                           std::move(cb));
+            body.addLambda(
+                "cb", "[args_on_heap = std::move(args_on_heap)](::hilti::rt::resumable::Handle* r) -> ::hilti::rt::any",
+                std::move(cb));
             body.addLocal({"r", "auto", {}, "std::make_unique<::hilti::rt::Resumable>(std::move(cb))"});
             body.addStatement("r->run()");
             body.addReturn("std::move(*r)");
@@ -586,14 +591,14 @@ struct GlobalsVisitor : hilti::visitor::PostOrder {
                 // Add a call to this to the module's initialization code.
                 cxx::Block call_init_func;
                 call_init_func.addStatement(fmt("%s()", d.id));
-                cg->unit()->addInitialization(call_init_func);
+                cg->unit()->addInitialization(std::move(call_init_func));
             }
 
             if ( n->linkage() == declaration::Linkage::PreInit ) {
                 // Add a call to this to the module's pre-initialization code.
                 cxx::Block call_preinit_func;
                 call_preinit_func.addStatement(fmt("%s()", d.id));
-                cg->unit()->addPreInitialization(call_preinit_func);
+                cg->unit()->addPreInitialization(std::move(call_preinit_func));
             }
         }
     }
@@ -699,6 +704,10 @@ std::vector<cxx::Expression> CodeGen::compileCallArguments(const node::Range<Exp
     for ( const auto& p : params ) {
         Expression* arg = (i < args.size() ? args[i] : p->default_());
         x.emplace_back(compile(arg, p->kind() == parameter::Kind::InOut));
+
+        if ( p->type()->type()->isA<type::Any>() )
+            x.emplace_back(typeInfo(arg->type()));
+
         i++;
     }
 
@@ -859,8 +868,8 @@ cxx::Expression CodeGen::unsignedIntegerToBitfield(type::Bitfield* t, const cxx:
     for ( const auto& b : t->bits(false) ) {
         auto x = fmt("::hilti::rt::integer::bits(%s, %d, %d, %s)", value, b->lower(), b->upper(), bitorder);
 
-        if ( auto a = b->attributes()->find(hilti::Attribute::Kind::Convert) ) {
-            pushDollarDollar(x);
+        if ( auto a = b->attributes()->find(hilti::attribute::kind::Convert) ) {
+            pushDollarDollar(std::move(x));
             bits.emplace_back(compile(*a->valueAsExpression()));
             popDollarDollar();
         }
@@ -875,6 +884,21 @@ cxx::Expression CodeGen::unsignedIntegerToBitfield(type::Bitfield* t, const cxx:
     return fmt("::hilti::rt::make_bitfield(%s)", util::join(bits, ", "));
 }
 
+std::pair<std::string, std::string> CodeGen::cxxTypeForVector(QualifiedType* element_type, bool want_iterator) {
+    auto etype = compile(element_type, codegen::TypeUsage::Storage);
+
+    std::string type_addl;
+
+    if ( want_iterator )
+        type_addl = (element_type->isConstant() ? "::const_iterator" : "::iterator");
+
+    if ( auto default_ = typeDefaultValue(element_type) )
+        return std::make_pair(fmt("::hilti::rt::Vector<%s, ::hilti::rt::vector::Allocator<%s>>%s", etype, etype,
+                                  type_addl),
+                              fmt(", {%s}", *default_));
+    else
+        return std::make_pair(fmt("::hilti::rt::Vector<%s>%s", etype, type_addl), std::string(""));
+}
 
 cxx::ID CodeGen::uniqueID(const std::string& prefix, Node* n) {
     if ( ! n->location() )
