@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <hilti/ast/expression.h>
 #include <hilti/ast/expressions/resolved-operator.h>
@@ -110,17 +110,17 @@ struct Visitor : hilti::visitor::PreOrder {
     /// Bitfield
 
     void operator()(operator_::bitfield::Member* n) final {
-        auto id = n->op1()->as<expression::Member>()->id();
+        const auto& id = n->op1()->as<expression::Member>()->id();
         auto elem = n->op0()->type()->type()->as<type::Bitfield>()->bitsIndex(id);
         assert(elem);
-        result = {fmt("(::hilti::rt::optional::value(std::get<%u>(%s.value)))", *elem, op0(n)), Side::RHS};
+        result = {fmt("(::hilti::rt::tuple::get<%u>(%s.value))", *elem, op0(n)), Side::RHS};
     }
 
     void operator()(operator_::bitfield::HasMember* n) final {
-        auto id = n->op1()->as<expression::Member>()->id();
+        const auto& id = n->op1()->as<expression::Member>()->id();
         auto elem = n->op0()->type()->type()->as<type::Bitfield>()->bitsIndex(id);
         assert(elem);
-        result = {fmt("std::get<%u>(%s.value).has_value()", *elem, op0(n)), Side::RHS};
+        result = {fmt("::hilti::rt::tuple::has_value<%u>(%s.value)", *elem, op0(n)), Side::RHS};
     }
 
     /// bytes::Iterator
@@ -152,7 +152,9 @@ struct Visitor : hilti::visitor::PreOrder {
     void operator()(operator_::bytes::SumAssignUInt8* n) final { result = fmt("%s.append(%s)", op0(n), op1(n)); }
     void operator()(operator_::bytes::Unequal* n) final { result = fmt("%s != %s", op0(n), op1(n)); }
 
-    void operator()(operator_::bytes::In* n) final { result = fmt("std::get<0>(%s.find(%s))", op1(n), op0(n)); }
+    void operator()(operator_::bytes::In* n) final {
+        result = fmt("::hilti::rt::tuple::get<0>(%s.find(%s))", op1(n), op0(n));
+    }
 
     void operator()(operator_::bytes::Find* n) final {
         auto [self, args] = methodArguments(n);
@@ -189,13 +191,18 @@ struct Visitor : hilti::visitor::PreOrder {
         result = fmt("%s.startsWith(%s)", self, args[0]);
     }
 
+    void operator()(operator_::bytes::EndsWith* n) final {
+        auto [self, args] = methodArguments(n);
+        result = fmt("%s.endsWith(%s)", self, args[0]);
+    }
+
     void operator()(operator_::bytes::Strip* n) final {
         auto [self, args] = methodArguments(n);
 
         std::string x;
 
         if ( auto side = optionalArgument(args, 1); ! side.empty() )
-            x = side;
+            x = std::move(side);
 
         if ( auto set = optionalArgument(args, 0); ! set.empty() ) {
             if ( x.size() )
@@ -346,7 +353,7 @@ struct Visitor : hilti::visitor::PreOrder {
 
         auto name = op0(n);
 
-        if ( auto a = f->function()->attributes()->find(hilti::Attribute::Kind::Cxxname) ) {
+        if ( auto a = f->function()->attributes()->find(hilti::attribute::kind::Cxxname) ) {
             if ( auto s = a->valueAsString() )
                 name = cxx::Expression(*s);
             else
@@ -544,9 +551,14 @@ struct Visitor : hilti::visitor::PreOrder {
             if ( auto x = ctor->tryAs<ctor::Coerced>() )
                 ctor = x->coercedCtor();
 
-            auto args = util::join(cg->compileCallArguments(ctor->as<ctor::Tuple>()->value(),
-                                                            tv->typeValue()->type()->parameters()),
-                                   ", ");
+            std::string args;
+
+            if ( ctor->as<ctor::Tuple>()->value().size() )
+                args = util::join(cg->compileCallArguments(ctor->as<ctor::Tuple>()->value(),
+                                                           tv->typeValue()->type()->parameters()),
+                                  ", ");
+            else if ( auto def = cg->typeDefaultValue(tv->typeValue()) )
+                args = *def;
 
             result = fmt("::hilti::rt::reference::make_strong<%s>(%s)",
                          cg->compile(tv->typeValue(), codegen::TypeUsage::Ctor), args);
@@ -665,10 +677,10 @@ struct Visitor : hilti::visitor::PreOrder {
     }
 
     void operator()(operator_::stream::view::InBytes* n) final {
-        result = fmt("std::get<0>(%s.find(%s))", op1(n), op0(n));
+        result = fmt("::hilti::rt::tuple::get<0>(%s.find(%s))", op1(n), op0(n));
     }
     void operator()(operator_::stream::view::InView* n) final {
-        result = fmt("std::get<0>(%s.find(%s))", op1(n), op0(n));
+        result = fmt("::hilti::rt::tuple::get<0>(%s.find(%s))", op1(n), op0(n));
     }
 
     void operator()(operator_::stream::view::AdvanceTo* n) final {
@@ -774,7 +786,7 @@ struct Visitor : hilti::visitor::PreOrder {
 
     void operator()(operator_::string::Encode* n) final {
         auto [self, args] = methodArguments(n);
-        result = fmt("::hilti::rt::Bytes(%s, %s)", self, args[0]);
+        result = fmt("::hilti::rt::string::encode(%s, %s, %s)", self, args[0], args[1]);
     }
 
     void operator()(operator_::string::Modulo* n) final {
@@ -815,6 +827,11 @@ struct Visitor : hilti::visitor::PreOrder {
         result = fmt("::hilti::rt::startsWith(%s, %s)", self, args[0]);
     }
 
+    void operator()(operator_::string::EndsWith* n) final {
+        auto [self, args] = methodArguments(n);
+        result = fmt("::hilti::rt::endsWith(%s, %s)", self, args[0]);
+    }
+
     void operator()(operator_::string::LowerCase* n) final { result = fmt("::hilti::rt::string::lower(%s)", op0(n)); }
 
     void operator()(operator_::string::UpperCase* n) final { result = fmt("::hilti::rt::string::upper(%s)", op0(n)); }
@@ -843,7 +860,7 @@ struct Visitor : hilti::visitor::PreOrder {
 
     cxx::Expression structMember(const expression::ResolvedOperator* o) {
         const auto& op0 = o->op0();
-        auto id = o->op1()->as<expression::Member>()->id();
+        const auto& id = o->op1()->as<expression::Member>()->id();
         auto attr = memberAccess(o, id);
 
         auto type = op0->type()->type();
@@ -866,7 +883,7 @@ struct Visitor : hilti::visitor::PreOrder {
             return fmt("::hilti::rt::optional::value(%s)", attr);
         }
 
-        return {attr, Side::LHS};
+        return {std::move(attr), Side::LHS};
     }
 
     void operator()(operator_::struct_::MemberConst* n) final { result = structMember(n); }
@@ -879,7 +896,7 @@ struct Visitor : hilti::visitor::PreOrder {
 
         auto ft = fdecl->type()->type()->as<type::Function>();
         auto args = n->op2()->as<expression::Ctor>()->ctor()->as<ctor::Tuple>()->value();
-        auto id = n->op1()->as<expression::Member>()->id();
+        const auto& id = n->op1()->as<expression::Member>()->id();
 
         std::vector<std::pair<Expression*, bool>> zipped;
 
@@ -898,7 +915,7 @@ struct Visitor : hilti::visitor::PreOrder {
     }
 
     void operator()(operator_::struct_::HasMember* n) final {
-        auto id = n->op1()->as<expression::Member>()->id();
+        const auto& id = n->op1()->as<expression::Member>()->id();
 
         if ( auto f = n->op0()->type()->type()->as<type::Struct>()->field(id); f->isOptional() )
             result = fmt("%s.has_value()", memberAccess(n, id));
@@ -907,7 +924,7 @@ struct Visitor : hilti::visitor::PreOrder {
     }
 
     void operator()(operator_::struct_::TryMember* n) final {
-        auto id = n->op1()->as<expression::Member>()->id();
+        const auto& id = n->op1()->as<expression::Member>()->id();
         assert(! lhs);
 
         if ( auto f = n->op0()->type()->type()->as<type::Struct>()->field(id); f->isOptional() ) {
@@ -930,7 +947,7 @@ struct Visitor : hilti::visitor::PreOrder {
     /// Union
 
     unsigned int unionFieldIndex(Expression* op0, Expression* op1) {
-        auto id = op1->as<expression::Member>()->id();
+        const auto& id = op1->as<expression::Member>()->id();
         return op0->type()->type()->as<type::Union>()->index(id);
     }
 
@@ -1091,7 +1108,7 @@ struct Visitor : hilti::visitor::PreOrder {
     void operator()(operator_::tuple::CustomAssign* n) final {
         auto t = n->operands()[0]->as<expression::Ctor>()->ctor()->as<ctor::Tuple>()->value();
         auto l = util::join(node::transform(t, [this](auto x) { return cg->compile(x, true); }), ", ");
-        result = {fmt("std::tie(%s) = %s", l, op1(n)), Side::LHS};
+        result = {fmt("::hilti::rt::tuple::assign(std::tie(%s), %s)", l, op1(n)), Side::LHS};
     }
 
     void operator()(operator_::tuple::Equal* n) final { result = fmt("%s == %s", op0(n), op1(n)); }
@@ -1099,14 +1116,14 @@ struct Visitor : hilti::visitor::PreOrder {
 
     void operator()(operator_::tuple::Index* n) final {
         auto i = n->op1()->as<expression::Ctor>()->ctor()->as<ctor::UnsignedInteger>()->value();
-        result = {fmt("std::get<%u>(%s)", i, op0(n)), Side::LHS};
+        result = {fmt("::hilti::rt::tuple::get<%u>(%s)", i, op0(n)), Side::LHS};
     }
 
     void operator()(operator_::tuple::Member* n) final {
-        auto id = n->op1()->as<expression::Member>()->id();
+        const auto& id = n->op1()->as<expression::Member>()->id();
         auto elem = n->op0()->type()->type()->as<type::Tuple>()->elementByID(id);
         assert(elem);
-        result = {fmt("std::get<%u>(%s)", elem->first, op0(n)), Side::LHS};
+        result = {fmt("::hilti::rt::tuple::get<%u>(%s)", elem->first, op0(n)), Side::LHS};
     }
 
     // Unsigned integer
@@ -1293,7 +1310,7 @@ struct Visitor : hilti::visitor::PreOrder {
 
 cxx::Expression CodeGen::compile(expression::ResolvedOperator* o, bool lhs) {
     auto v = Visitor(this, lhs);
-    if ( auto x = hilti::visitor::dispatch(v, o, [](const auto& v) { return v.result; }) )
+    if ( auto x = hilti::visitor::dispatch(v, o, [](const auto& v) -> const auto& { return v.result; }) )
         return lhs ? _makeLhs(*x, o->type()) : *x;
 
     std::cerr << o->dump();
