@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <optional>
 
@@ -13,7 +13,6 @@
 #include <hilti/compiler/detail/cxx/all.h>
 #include <hilti/compiler/unit.h>
 
-
 using namespace hilti;
 using namespace hilti::detail;
 using namespace hilti::detail::codegen;
@@ -98,8 +97,8 @@ struct VisitorDeclaration : hilti::visitor::PreOrder {
                     else
                         default_ = cg->typeDefaultValue(p->type());
 
-                    auto arg = cxx::declaration::Argument(cxx::ID(fmt("__p_%s", p->id())), type, std::move(default_),
-                                                          internal_type);
+                    auto arg = cxx::declaration::Argument(cxx::ID(fmt("__p_%s", p->id())), std::move(type),
+                                                          std::move(default_), std::move(internal_type));
                     args.emplace_back(std::move(arg));
                 }
 
@@ -187,9 +186,9 @@ struct VisitorDeclaration : hilti::visitor::PreOrder {
                             }
 
                             d.ftype = cxx::declaration::Function::Free;
-                            auto hook = cxx::linker::Join{.id = cxx::ID(id_hook),
+                            auto hook = cxx::linker::Join{.id = cxx::ID(std::move(id_hook)),
                                                           .callee = d,
-                                                          .aux_types = aux_types,
+                                                          .aux_types = std::move(aux_types),
                                                           .declare_only = true};
 
                             auto vref =
@@ -234,8 +233,8 @@ struct VisitorDeclaration : hilti::visitor::PreOrder {
                     // Do not pass a default value here since initialization of
                     // the member happens (and needs to happen) via the ctor
                     // block above to guarantee we have a block.
-                    auto x =
-                        cxx::declaration::Local(cxx::ID(f->id()), t, {}, {}, (f->isStatic() ? "inline static" : ""));
+                    auto x = cxx::declaration::Local(cxx::ID(f->id()), std::move(t), {}, {},
+                                                     (f->isStatic() ? "inline static" : ""));
 
 
                     fields.emplace_back(std::move(x));
@@ -289,12 +288,12 @@ struct VisitorDeclaration : hilti::visitor::PreOrder {
         std::vector<cxx::type::union_::Member> fields;
         for ( const auto& f : n->fields() ) {
             auto t = cg->compile(f->type(), codegen::TypeUsage::Storage);
-            auto x = cxx::declaration::Local(cxx::ID(f->id()), t);
+            auto x = cxx::declaration::Local(cxx::ID(f->id()), std::move(t));
             fields.emplace_back(std::move(x));
         }
 
         auto t = cxx::type::Union{.members = std::move(fields), .type_name = cxx::ID(id.local())};
-        result = cxx::declaration::Type(id, t);
+        result = cxx::declaration::Type(std::move(id), t);
     }
 
     void operator()(type::Vector* n) final {
@@ -314,7 +313,7 @@ struct VisitorDeclaration : hilti::visitor::PreOrder {
         auto id = cxx::ID(scope, sid);
         auto labels = util::transform(n->labels(), [](auto l) { return std::make_pair(cxx::ID(l->id()), l->value()); });
         auto t = cxx::type::Enum{.labels = std::move(labels), .type_name = cxx::ID(id.local())};
-        auto decl = cxx::declaration::Type(id, t, {}, true);
+        auto decl = cxx::declaration::Type(std::move(id), t, {}, true);
         dependencies.push_back(decl);
         result = decl;
     }
@@ -465,9 +464,8 @@ struct VisitorStorage : hilti::visitor::PreOrder {
     }
 
     void operator()(type::list::Iterator* n) final {
-        auto t =
-            fmt("::hilti::rt::Vector<%s>::iterator_t", cg->compile(n->dereferencedType(), codegen::TypeUsage::Storage));
-        result = CxxTypes{.base_type = fmt("%s", t)};
+        auto [cxx_type, _] = cg->cxxTypeForVector(n->dereferencedType(), true);
+        result = CxxTypes{.base_type = cxx_type};
     }
 
     void operator()(type::map::Iterator* n) final {
@@ -489,29 +487,23 @@ struct VisitorStorage : hilti::visitor::PreOrder {
 
 
     void operator()(type::vector::Iterator* n) final {
-        auto i = (n->dereferencedType()->isConstant() ? "const_iterator" : "iterator");
-        auto x = cg->compile(n->dereferencedType(), codegen::TypeUsage::Storage);
-
-        std::string allocator;
-        if ( auto def = cg->typeDefaultValue(n->dereferencedType()) )
-            allocator = fmt(", ::hilti::rt::vector::Allocator<%s, %s>", x, *def);
-
-        auto t = fmt("::hilti::rt::Vector<%s%s>::%s", x, allocator, i);
-        result = CxxTypes{.base_type = fmt("%s", t)};
+        auto [cxx_type, _] = cg->cxxTypeForVector(n->dereferencedType(), true);
+        result = CxxTypes{.base_type = cxx_type};
     }
 
-    void operator()(type::Library* n) final { result = CxxTypes{.base_type = fmt("%s", n->cxxName())}; }
+    void operator()(type::Library* n) final {
+        result = CxxTypes{.base_type = fmt("%s%s", (n->isConstant() ? "const " : ""), n->cxxName()),
+                          .param_in = fmt("const %s %s &", n->cxxName(), (n->isConstant() ? "const " : ""))};
+    }
 
     void operator()(type::List* n) final {
-        std::string t;
-
         if ( n->elementType()->type()->isA<type::Unknown>() )
             // Can only be the empty list.
-            t = "::hilti::rt::vector::Empty";
-        else
-            t = fmt("::hilti::rt::Vector<%s>", cg->compile(n->elementType(), codegen::TypeUsage::Storage));
-
-        result = CxxTypes{.base_type = fmt("%s", t)};
+            result = CxxTypes{.base_type = "::hilti::rt::list::Empty"};
+        else {
+            auto [cxx_type, _] = cg->cxxTypeForVector(n->elementType());
+            result = CxxTypes{.base_type = cxx_type};
+        }
     }
 
     void operator()(type::Map* n) final {
@@ -602,22 +594,13 @@ struct VisitorStorage : hilti::visitor::PreOrder {
     }
 
     void operator()(type::Vector* n) final {
-        std::string t;
-
         if ( n->elementType()->type()->isA<type::Unknown>() )
             // Can only be the empty list.
-            t = "::hilti::rt::vector::Empty";
+            result = CxxTypes{.base_type = "::hilti::rt::vector::Empty"};
         else {
-            auto x = cg->compile(n->elementType(), codegen::TypeUsage::Storage);
-
-            std::string allocator;
-            if ( auto def = cg->typeDefaultValue(n->elementType()) )
-                allocator = fmt(", ::hilti::rt::vector::Allocator<%s, %s>", x, *def);
-
-            t = fmt("::hilti::rt::Vector<%s%s>", x, allocator);
+            auto [cxx_type, _] = cg->cxxTypeForVector(n->elementType());
+            result = CxxTypes{.base_type = cxx_type};
         }
-
-        result = CxxTypes{.base_type = fmt("%s", t)};
     }
 
     void operator()(type::Time* n) final { result = CxxTypes{.base_type = "::hilti::rt::Time"}; }
@@ -718,10 +701,28 @@ struct VisitorStorage : hilti::visitor::PreOrder {
     }
 
     void operator()(type::Tuple* n) final {
-        auto x = node::transform(n->elements(),
-                                 [this](auto e) { return cg->compile(e->type(), codegen::TypeUsage::Storage); });
-        auto t = fmt("std::tuple<%s>", util::join(x, ", "));
-        result = CxxTypes{.base_type = t};
+        auto types =
+            util::join(util::transform(n->elements(),
+                                       [this](auto e) { return cg->compile(e->type(), codegen::TypeUsage::Storage); }),
+                       ", ");
+
+        auto defaults =
+            util::join(util::transform(n->elements(),
+                                       [this](auto e) {
+                                           if ( auto d = cg->typeDefaultValue(e->type()) )
+                                               return fmt("{%s}", *d);
+                                           else
+                                               // Would prefer to just to return "{{}}" here, but that doesn't work
+                                               // on some compilers; seen for example on CentOS Stream 9 and Ubuntu 24.
+                                               // Not sure if that's expected or not.
+                                               return fmt("std::make_optional<%s>({})",
+                                                          cg->compile(e->type(), codegen::TypeUsage::Storage));
+                                       }),
+                       ", ");
+
+        auto base_type = fmt("::hilti::rt::Tuple<%s>", types);
+        auto default_ = fmt("::hilti::rt::Tuple<%s>{%s}", types, defaults);
+        result = CxxTypes{.base_type = base_type, .default_ = default_};
     }
 
     void operator()(type::Name* n) final {
@@ -770,6 +771,7 @@ struct VisitorTypeInfoPredefined : hilti::visitor::PreOrder {
     void operator()(type::Error* n) final { result = "::hilti::rt::type_info::error"; }
     void operator()(type::Interval* n) final { result = "::hilti::rt::type_info::interval"; }
     void operator()(type::Network* n) final { result = "::hilti::rt::type_info::network"; }
+    void operator()(type::Null* n) final { result = "::hilti::rt::type_info::null"; }
     void operator()(type::Port* n) final { result = "::hilti::rt::type_info::port"; }
     void operator()(type::Real* n) final { result = "::hilti::rt::type_info::real"; }
     void operator()(type::RegExp* n) final { result = "::hilti::rt::type_info::regexp"; }
@@ -804,9 +806,10 @@ struct VisitorTypeInfoDynamic : hilti::visitor::PreOrder {
 
         auto i = 0;
         for ( const auto&& b : n->bits() )
-            elems.push_back(fmt(
-                "::hilti::rt::type_info::bitfield::Bits{ \"%s\", %s, ::hilti::rt::bitfield::elementOffset<%s, %d>() }",
-                b->id(), cg->typeInfo(b->itemType()), ttype, i++));
+            elems.push_back(
+                fmt("::hilti::rt::type_info::bitfield::Bits{ \"%s\", %s, ::hilti::rt::bitfield::elementOffset<%s, "
+                    "%d>() }",
+                    b->id(), cg->typeInfo(b->itemType()), ttype, i++));
 
         result = fmt("::hilti::rt::type_info::Bitfield(std::vector<::hilti::rt::type_info::bitfield::Bits>({%s}))",
                      util::join(elems, ", "));
@@ -826,7 +829,33 @@ struct VisitorTypeInfoDynamic : hilti::visitor::PreOrder {
 
     void operator()(type::Function* n) final { result = "::hilti::rt::type_info::Function()"; }
 
-    void operator()(type::Library* n) final { result = "::hilti::rt::type_info::Library()"; }
+    void operator()(type::Library* n) final { result = fmt("::hilti::rt::type_info::Library(\"%s\")", n->cxxName()); }
+
+    // Helper factoring out common logic for creating type information for
+    // vectors and vector iterators.
+    std::string typeInfoForVector(QualifiedType* element_type, bool want_iterator = false) {
+        auto etype = cg->compile(element_type, codegen::TypeUsage::Storage);
+        auto type_name = (want_iterator ? "VectorIterator" : "Vector");
+
+        std::string type_addl;
+
+        if ( want_iterator )
+            type_addl = (element_type->isConstant() ? "::const_iterator" : "::iterator");
+
+        if ( auto default_ = cg->typeDefaultValue(element_type) )
+            return fmt(
+                "::hilti::rt::type_info::%s(%s, ::hilti::rt::type_info::%s::accessor<%s, "
+                "::hilti::rt::vector::Allocator<%s>>())",
+                type_name, cg->typeInfo(element_type), type_name, etype, etype);
+        else
+            return fmt("::hilti::rt::type_info::%s(%s, ::hilti::rt::type_info::%s::accessor<%s>())", type_name,
+                       cg->typeInfo(element_type), type_name, etype);
+    }
+
+    void operator()(type::List* n) final {
+        // This generates type information for a vector, as that's how we store lists.
+        result = typeInfoForVector(n->elementType());
+    }
 
     void operator()(type::Map* n) final {
         auto ktype = cg->compile(n->keyType(), codegen::TypeUsage::Storage);
@@ -852,9 +881,12 @@ struct VisitorTypeInfoDynamic : hilti::visitor::PreOrder {
     }
 
     void operator()(type::Result* n) final {
-        result =
-            fmt("::hilti::rt::type_info::Result(%s, ::hilti::rt::type_info::Result::accessor<%s>())",
-                cg->typeInfo(n->dereferencedType()), cg->compile(n->dereferencedType(), codegen::TypeUsage::Storage));
+        if ( ! n->dereferencedType()->type()->isA<type::Void>() )
+            result = fmt("::hilti::rt::type_info::Result(%s, ::hilti::rt::type_info::Result::accessor<%s>())",
+                         cg->typeInfo(n->dereferencedType()),
+                         cg->compile(n->dereferencedType(), codegen::TypeUsage::Storage));
+        else
+            result = fmt("::hilti::rt::type_info::Result(%s, {})", cg->typeInfo(n->dereferencedType()));
     }
 
     void operator()(type::Set* n) final {
@@ -905,7 +937,7 @@ struct VisitorTypeInfoDynamic : hilti::visitor::PreOrder {
         for ( const auto& e : n->elements() ) {
             elems.push_back(
                 fmt("::hilti::rt::type_info::tuple::Element{ \"%s\", %s, ::hilti::rt::tuple::elementOffset<%s, %d>() }",
-                    e->id() ? e->id() : ID(), cg->typeInfo(e->type()), ttype, i));
+                    e->id() ? e->id() : ID(), cg->typeInfo(e->typeWithOptional()), ttype, i));
             ++i;
         }
 
@@ -943,28 +975,9 @@ struct VisitorTypeInfoDynamic : hilti::visitor::PreOrder {
                 cg->typeInfo(n->dereferencedType()), cg->compile(n->dereferencedType(), codegen::TypeUsage::Storage));
     }
 
-    void operator()(type::Vector* n) final {
-        auto x = cg->compile(n->elementType(), codegen::TypeUsage::Storage);
+    void operator()(type::Vector* n) final { result = typeInfoForVector(n->elementType()); }
 
-        std::string allocator;
-        if ( auto def = cg->typeDefaultValue(n->elementType()) )
-            allocator = fmt(", ::hilti::rt::vector::Allocator<%s, %s>", x, *def);
-
-        result = fmt("::hilti::rt::type_info::Vector(%s, ::hilti::rt::type_info::Vector::accessor<%s%s>())",
-                     cg->typeInfo(n->elementType()), x, allocator);
-    }
-
-    void operator()(type::vector::Iterator* n) final {
-        auto x = cg->compile(n->dereferencedType(), codegen::TypeUsage::Storage);
-
-        std::string allocator;
-        if ( auto def = cg->typeDefaultValue(n->dereferencedType()) )
-            allocator = fmt(", ::hilti::rt::vector::Allocator<%s, %s>", x, *def);
-
-        result =
-            fmt("::hilti::rt::type_info::VectorIterator(%s, ::hilti::rt::type_info::VectorIterator::accessor<%s%s>())",
-                cg->typeInfo(n->dereferencedType()), x, allocator);
-    }
+    void operator()(type::vector::Iterator* n) final { result = typeInfoForVector(n->dereferencedType(), true); }
 
     void operator()(type::Name* n) final {
         assert(n->resolvedTypeIndex());
@@ -978,7 +991,7 @@ struct VisitorTypeInfoDynamic : hilti::visitor::PreOrder {
 
 cxx::Type CodeGen::compile(QualifiedType* t, codegen::TypeUsage usage) {
     auto v = VisitorStorage(this, t, &_cache_types_storage, usage);
-    auto x = hilti::visitor::dispatch(v, t->type(), [](const auto& v) { return v.result; });
+    auto x = hilti::visitor::dispatch(v, t->type(), [](const auto& v) -> const auto& { return v.result; });
     if ( ! x ) {
         std::cerr << t->dump();
         logger().internalError(fmt("codegen: type %s does not have a visitor", *t), t);
@@ -1068,7 +1081,7 @@ cxx::Type CodeGen::compile(QualifiedType* t, codegen::TypeUsage usage) {
 
 std::optional<cxx::Expression> CodeGen::typeDefaultValue(hilti::QualifiedType* t) {
     auto v = VisitorStorage(this, t, &_cache_types_storage, codegen::TypeUsage::None);
-    auto x = hilti::visitor::dispatch(v, t->type(), [](const auto& v) { return v.result; });
+    auto x = hilti::visitor::dispatch(v, t->type(), [](const auto& v) -> const auto& { return v.result; });
     if ( ! x ) {
         std::cerr << t->dump();
         logger().internalError(fmt("codegen: type %s does not have a visitor", *t), t);
@@ -1088,7 +1101,7 @@ std::optional<cxx::declaration::Type> CodeGen::typeDeclaration(QualifiedType* t)
         return {};
 
     auto v = VisitorDeclaration(this, t, &_cache_types_declarations);
-    return hilti::visitor::dispatch(v, t->type(), [](const auto& v) { return v.result; });
+    return hilti::visitor::dispatch(v, t->type(), [](const auto& v) -> const auto& { return v.result; });
 };
 
 const CxxTypeInfo& CodeGen::_getOrCreateTypeInfo(QualifiedType* t) {
@@ -1116,7 +1129,8 @@ const CxxTypeInfo& CodeGen::_getOrCreateTypeInfo(QualifiedType* t) {
         [&]() {
             auto v = VisitorTypeInfoPredefined(this);
 
-            if ( auto x = hilti::visitor::dispatch(v, t->type(), [](const auto& v) { return v.result; }); x && *x )
+            if ( auto x = hilti::visitor::dispatch(v, t->type(), [](const auto& v) -> const auto& { return v.result; });
+                 x && *x )
                 return CxxTypeInfo{.predefined = true, .reference = fmt("&%s", *x)};
 
             auto forward = cxx::declaration::Constant(tid, "::hilti::rt::TypeInfo", {}, "extern");
@@ -1129,13 +1143,24 @@ const CxxTypeInfo& CodeGen::_getOrCreateTypeInfo(QualifiedType* t) {
                 return ti;
 
             auto v = VisitorTypeInfoDynamic(this, t);
-            auto x = hilti::visitor::dispatch(v, t->type(), [](const auto& v) { return v.result; });
+            auto x = hilti::visitor::dispatch(v, t->type(), [](const auto& v) -> const auto& { return v.result; });
             if ( ! x )
                 logger().internalError(fmt("codegen: type %s does not have a dynamic type info visitor", *t), t);
 
-            auto id_init = (t->type()->typeID() ? fmt("\"%s\"", t->type()->typeID()) : std::string("{}"));
-            auto init = fmt("{ %s, \"%s\", new %s }", id_init,
-                            util::escapeUTF8(display.str(), util::render_style::UTF8::EscapeQuotes), *x);
+            const auto& id_init = (t->type()->typeID() ? fmt("\"%s\"", t->type()->typeID()) : std::string("{}"));
+
+            std::string to_string;
+
+            if ( auto x = t->type()->tryAs<type::Library>() )
+                // Library types cannot be rendered into strings, just hardcode the type's name.
+                to_string = fmt("[](const void *self) { return \"<%s>\"s; }", x->cxxName());
+            else
+                to_string =
+                    fmt("[](const void *self) { return hilti::rt::to_string(*reinterpret_cast<const %s*>(self)); }",
+                        compile(t, codegen::TypeUsage::Storage));
+
+            auto init = fmt("{ %s, \"%s\", %s, new %s }", id_init,
+                            util::escapeUTF8(display.str(), util::render_style::UTF8::EscapeQuotes), to_string, *x);
 
             ti.declaration = cxx::declaration::Constant(tid, "::hilti::rt::TypeInfo", init, "");
 
