@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <optional>
 #include <string>
@@ -58,16 +58,16 @@ struct Visitor : hilti::visitor::PreOrder {
         if ( args.size() > 1 )
             bitorder = args[1];
 
-        auto unpacked =
-            cg->addTmp("x",
-                       cxx::Type(util::fmt("::hilti::rt::Result<std::tuple<::hilti::rt::integer::safe<uint%d_t>, %s>>",
-                                           n->width(), cg->compile(data_type, codegen::TypeUsage::Storage))));
+        auto unpacked = cg->addTmp(
+            "x",
+            cxx::Type(util::fmt("::hilti::rt::Result<::hilti::rt::Tuple<::hilti::rt::integer::safe<uint%d_t>, %s>>",
+                                n->width(), cg->compile(data_type, codegen::TypeUsage::Storage))));
         auto unpack_uint =
             fmt("%s = ::hilti::rt::integer::unpack<uint%d_t>(%s, %s)", unpacked, n->width(), data, args[0]);
 
-        auto bf_value = cg->unsignedIntegerToBitfield(n, fmt("std::get<0>(*%s)", unpacked), bitorder);
-        result = fmt("(%s, ::hilti::rt::make_result(std::make_tuple(%s, std::get<1>(*%s))))", unpack_uint, bf_value,
-                     unpacked);
+        auto bf_value = cg->unsignedIntegerToBitfield(n, fmt("::hilti::rt::tuple::get<0>(*%s)", unpacked), bitorder);
+        result = fmt("(%s, ::hilti::rt::make_result(::hilti::rt::tuple::make(%s, ::hilti::rt::tuple::get<1>(*%s))))",
+                     unpack_uint, bf_value, unpacked);
     }
 
     void operator()(type::UnsignedInteger* n) final {
@@ -88,7 +88,8 @@ struct Visitor : hilti::visitor::PreOrder {
 cxx::Expression CodeGen::pack(Expression* data, const Expressions& args) {
     auto cxx_args = util::transform(args, [&](const auto& e) { return compile(e, false); });
     auto v = Visitor(this, Visitor::Kind::Pack, nullptr, compile(data), cxx_args);
-    if ( auto result = hilti::visitor::dispatch(v, data->type()->type(), [](const auto& v) { return v.result; }) )
+    if ( auto result =
+             hilti::visitor::dispatch(v, data->type()->type(), [](const auto& v) -> const auto& { return v.result; }) )
         return cxx::Expression(*result);
 
     logger().internalError("pack failed to compile", data->type());
@@ -96,7 +97,7 @@ cxx::Expression CodeGen::pack(Expression* data, const Expressions& args) {
 
 cxx::Expression CodeGen::pack(QualifiedType* t, const cxx::Expression& data, const std::vector<cxx::Expression>& args) {
     auto v = Visitor(this, Visitor::Kind::Pack, nullptr, data, args);
-    if ( auto result = hilti::visitor::dispatch(v, t->type(), [](const auto& v) { return v.result; }) )
+    if ( auto result = hilti::visitor::dispatch(v, t->type(), [](const auto& v) -> const auto& { return v.result; }) )
         return cxx::Expression(*result);
 
     logger().internalError("pack failed to compile", t);
@@ -106,7 +107,7 @@ cxx::Expression CodeGen::unpack(QualifiedType* t, QualifiedType* data_type, Expr
                                 bool throw_on_error) {
     auto cxx_args = util::transform(args, [&](const auto& e) { return compile(e, false); });
     auto v = Visitor(this, Visitor::Kind::Unpack, data_type, compile(data), cxx_args);
-    if ( auto result = hilti::visitor::dispatch(v, t->type(), [](const auto& v) { return v.result; }) ) {
+    if ( auto result = hilti::visitor::dispatch(v, t->type(), [](const auto& v) -> const auto& { return v.result; }) ) {
         if ( throw_on_error )
             return cxx::Expression(util::fmt("%s.valueOrThrow()", *result));
         else
@@ -119,7 +120,7 @@ cxx::Expression CodeGen::unpack(QualifiedType* t, QualifiedType* data_type, Expr
 cxx::Expression CodeGen::unpack(QualifiedType* t, QualifiedType* data_type, const cxx::Expression& data,
                                 const std::vector<cxx::Expression>& args, bool throw_on_error) {
     auto v = Visitor(this, Visitor::Kind::Unpack, data_type, data, args);
-    if ( auto result = hilti::visitor::dispatch(v, t->type(), [](const auto& v) { return v.result; }) ) {
+    if ( auto result = hilti::visitor::dispatch(v, t->type(), [](const auto& v) -> const auto& { return v.result; }) ) {
         if ( throw_on_error )
             return cxx::Expression(util::fmt("%s.valueOrThrow<::hilti::rt::InvalidValue>()", *result));
         else
