@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <utility>
 
@@ -60,7 +60,7 @@ struct VisitorCtor : visitor::PreOrder {
                     return;
             }
 
-            result = builder->ctorMap(t->keyType(), t->elementType(), nelemns, n->meta());
+            result = builder->ctorMap(t->keyType(), t->elementType(), std::move(nelemns), n->meta());
         }
     }
 
@@ -122,7 +122,7 @@ struct VisitorCtor : visitor::PreOrder {
                 else
                     return;
             }
-            result = builder->ctorSet(dt, nexprs, n->meta());
+            result = builder->ctorSet(dt, std::move(nexprs), n->meta());
         }
     }
 
@@ -579,7 +579,10 @@ struct VisitorType : visitor::PreOrder {
     }
 
     void operator()(type::Type_* n) final {
-        if ( auto t = dst->type()->tryAs<type::Type_>() ) {
+        if ( auto lt = dst->type()->tryAs<type::Library>(); lt && lt->cxxName() == "::hilti::rt::TypeInfo*" )
+            result = dst->recreateAsConst(builder->context());
+
+        else if ( auto t = dst->type()->tryAs<type::Type_>() ) {
             if ( type::sameExceptForConstness(n->typeValue(), t->typeValue()) )
                 result = src;
         }
@@ -927,7 +930,7 @@ Result<std::pair<bool, Expressions>> hilti::coerceOperands(Builder* builder, ope
 
 // If an expression is a reference, dereference it; otherwise return the
 // expression itself.
-Expression* skipReferenceValue(Builder* builder, Expression* op) {
+static Expression* skipReferenceValue(Builder* builder, Expression* op) {
     static auto value_reference_deref = operator_::get("value_reference::Deref");
     static auto strong_reference_deref = operator_::get("strong_reference::Deref");
     static auto weak_reference_deref = operator_::get("weak_reference::Deref");
