@@ -12,7 +12,6 @@
 #include <hilti/ast/declarations/imported-module.h>
 #include <hilti/ast/declarations/local-variable.h>
 #include <hilti/ast/declarations/parameter.h>
-#include <hilti/ast/expressions/deferred.h>
 #include <hilti/ast/expressions/keyword.h>
 #include <hilti/ast/expressions/list-comprehension.h>
 #include <hilti/ast/expressions/name.h>
@@ -166,10 +165,10 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
             }
 
             if ( ! hook_id ) {
-                hook_id = canon_id;
+                hook_id = std::move(canon_id);
                 hook_type = ftype;
             }
-            else {
+            else if ( hook_type ) {
                 if ( canon_id != hook_id || ! type::same(ftype, hook_type) )
                     return false;
             }
@@ -291,7 +290,7 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
     // expressions or the type aren't fully resolved yet. Returns an error if a
     // coercion failed with a hard error.
     template<typename Container1, typename Container2>
-    Result<std::optional<Expressions>> coerceCallArguments(Container1 exprs, Container2 params) {
+    Result<std::optional<Expressions>> coerceCallArguments(Container1 exprs, const Container2& params) {
         // Build a tuple to coerce expression according to an OperandList.
         for ( const auto& e : exprs ) {
             if ( ! e->isResolved() )
@@ -537,11 +536,11 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
     }
 
     void operator()(Attribute* n) final {
-        if ( n->kind() == hilti::Attribute::Kind::Cxxname && n->hasValue() ) {
+        if ( n->kind() == hilti::attribute::kind::Cxxname && n->hasValue() ) {
             // Normalize values passed as `&cxxname` so they always are interpreted as FQNs by enforcing leading
             // `::`.
             if ( const auto& value = n->valueAsString(); value && ! util::startsWith(*value, "::") ) {
-                auto a = builder()->attribute(hilti::Attribute::Kind::Cxxname,
+                auto a = builder()->attribute(hilti::attribute::kind::Cxxname,
                                               builder()->stringLiteral(util::fmt("::%s", *value)));
                 replaceNode(n, a);
             }
@@ -554,7 +553,7 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
                 if ( auto x = n->typeArguments(); x.size() ) {
                     if ( auto coerced = coerceCallArguments(x, t->type()->parameters()); coerced && *coerced ) {
                         recordChange(n, builder()->ctorTuple(**coerced), "call arguments");
-                        n->setTypeArguments(context(), std::move(**coerced));
+                        n->setTypeArguments(context(), **coerced);
                     }
                 }
             }
@@ -811,12 +810,12 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
             recordChange(n, util::fmt("set linked type to %s", index));
         }
 
-        if ( auto a = n->attributes()->find(hilti::Attribute::Kind::Default) ) {
+        if ( auto a = n->attributes()->find(hilti::attribute::kind::Default) ) {
             auto val = a->valueAsExpression();
             if ( auto x = coerceTo(n, *val, n->type(), false, true) ) {
                 recordChange(*val, x, "attribute");
-                n->attributes()->remove(hilti::Attribute::Kind::Default);
-                n->attributes()->add(context(), builder()->attribute(hilti::Attribute::Kind::Default, x));
+                n->attributes()->remove(hilti::attribute::kind::Default);
+                n->attributes()->add(context(), builder()->attribute(hilti::attribute::kind::Default, x));
             }
         }
 
@@ -848,7 +847,7 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
             declaration::Type* linked_type = nullptr;
             Declaration* linked_prototype = nullptr;
 
-            if ( auto resolved = scope::lookupID<declaration::Type>(ns, n, "struct type") ) {
+            if ( auto resolved = scope::lookupID<declaration::Type>(std::move(ns), n, "struct type") ) {
                 linked_type = resolved->first;
 
                 for ( const auto& field : linked_type->type()->type()->as<type::Struct>()->fields(n->id().local()) ) {
@@ -991,7 +990,7 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
         Expression* init = nullptr;
         std::optional<Expressions> args;
 
-        if ( auto e = n->init() ) {
+        if ( auto e = n->init(); e && ! e->isA<expression::Void>() ) {
             if ( auto x = coerceTo(n, e, n->type(), false, true) )
                 init = x;
         }
@@ -1075,9 +1074,9 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
         }
 
         if ( auto x = n->type()->type()->tryAs<type::Library>();
-             x && ! n->attributes()->has(hilti::Attribute::Kind::Cxxname) )
+             x && ! n->attributes()->has(hilti::attribute::kind::Cxxname) )
             // Transfer the C++ name into an attribute.
-            n->attributes()->add(context(), builder()->attribute(hilti::Attribute::Kind::Cxxname,
+            n->attributes()->add(context(), builder()->attribute(hilti::attribute::kind::Cxxname,
                                                                  builder()->stringLiteral(x->cxxName())));
     }
 
@@ -1148,13 +1147,6 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
         }
     }
 
-    void operator()(expression::Deferred* n) final {
-        if ( ! n->type()->isResolved() && n->expression()->isResolved() ) {
-            recordChange(n, n->expression()->type());
-            n->setType(context(), n->expression()->type());
-        }
-    }
-
     void operator()(expression::Keyword* n) final {
         if ( n->kind() == expression::keyword::Kind::Scope && ! n->type()->isResolved() ) {
             auto ntype = builder()->qualifiedType(builder()->typeString(), Constness::Const);
@@ -1226,7 +1218,7 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
             else
                 id = n->id();
 
-            auto resolved = scope::lookupID<Declaration>(id, scope_node, "declaration");
+            auto resolved = scope::lookupID<Declaration>(std::move(id), scope_node, "declaration");
             if ( resolved ) {
                 auto index = context()->register_(resolved->first);
                 n->setResolvedDeclarationIndex(context(), index);
@@ -1360,7 +1352,7 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
 
             if ( n->kind() == operator_::Kind::MemberCall ) {
                 if ( auto stype = match->op0()->type()->type()->tryAs<type::Struct>() ) {
-                    auto id = match->op1()->as<expression::Member>()->id();
+                    const auto& id = match->op1()->as<expression::Member>()->id();
                     if ( auto field = stype->field(id) ) {
                         auto ftype = field->type()->type()->as<type::Function>();
                         recordAutoParameters(*ftype, match->op2());
@@ -1582,7 +1574,7 @@ struct VisitorPass2 : visitor::MutatingPostOrder {
         if ( ! type::isResolved(n->itemType()) ) {
             auto t = n->ddType();
 
-            if ( auto a = n->attributes()->find(hilti::Attribute::Kind::Convert) )
+            if ( auto a = n->attributes()->find(hilti::attribute::kind::Convert) )
                 t = (*a->valueAsExpression())->type();
 
             if ( t->isResolved() ) {
