@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <hilti/ast/ast-context.h>
 #include <hilti/ast/builder/builder.h>
@@ -187,7 +187,7 @@ public:
         if ( unifier.isAborted() )
             return;
 
-        auto serial = unifier.serialization();
+        const auto& serial = unifier.serialization();
         if ( serial.empty() ) {
             std::cerr << n->dump();
             logger().internalError("empty type _serialization for unification, type not implemented?");
