@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <algorithm>
 #include <unordered_set>
@@ -60,6 +60,18 @@
 using namespace hilti;
 using util::fmt;
 
+/**
+ * A mapping of node tags to any attributes that node allows. When a new
+ * attribute is added, this map must be updated to accept that attribute on any
+ * nodes it applies to. These checks are applied only to actual HILTI modules.
+ */
+static std::unordered_map<node::Tag, std::unordered_set<attribute::Kind>> allowed_attributes{
+    {node::tag::Function,
+     {attribute::kind::Cxxname, attribute::kind::HavePrototype, attribute::kind::Priority, attribute::kind::Static,
+      attribute::kind::NeededByFeature, attribute::kind::Debug}},
+    {node::tag::declaration::Parameter, {attribute::kind::RequiresTypeFeature}},
+};
+
 void hilti::validator::VisitorMixIn::deprecated(const std::string& msg, const Location& l) const {
     hilti::logger().deprecated(msg, l);
 }
@@ -124,6 +136,34 @@ struct VisitorPre : visitor::PreOrder, public validator::VisitorMixIn {
 struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
     using hilti::validator::VisitorMixIn::VisitorMixIn;
 
+    // Ensures that the node represented by tag is allowed to have all of the
+    // provided attributes. This does not use any context, if more information
+    // is needed, then do the check elsewhere.
+    void checkNodeAttributes(Node* n, AttributeSet* attributes, const std::string_view& where) {
+        if ( ! attributes )
+            return;
+
+        if ( auto current_module = n->parent<declaration::Module>();
+             current_module && current_module->uid().process_extension != ".hlt" )
+            return;
+
+        auto it = allowed_attributes.find(n->nodeTag());
+
+        if ( it == allowed_attributes.end() ) {
+            if ( ! attributes->attributes().empty() )
+                error(hilti::util::fmt("No attributes expected in %s", where), attributes);
+
+            return;
+        }
+
+        auto allowed = it->second;
+
+        for ( const auto& attr : attributes->attributes() ) {
+            if ( allowed.find(attr->kind()) == allowed.end() )
+                error(hilti::util::fmt("invalid attribute '%s' in %s", to_string(attr->kind()), where), attr);
+        }
+    }
+
     // Returns an error if the given type cannot be used for ordering at
     // runtime.
     Result<Nothing> isSortable(QualifiedType* t) {
@@ -141,6 +181,16 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
         return Nothing();
     }
 
+    // Ensures the declaration's type is a valid type.
+    void checkDeclarationType(Declaration* decl, QualifiedType* ty) {
+        if ( ty->type()->isA<hilti::type::Struct>() || ty->type()->isA<hilti::type::Enum>() ||
+             ty->type()->isA<hilti::type::Union>() ) {
+            if ( ! ty->type()->typeID() )
+                error(fmt("%s types must be named in declarations", ty->type()->typeClass()), decl,
+                      node::ErrorPriority::High);
+        }
+    }
+
     void operator()(Node* n) final {
         if ( ! n->scope() )
             return;
@@ -150,18 +200,36 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
             if ( nodes.size() <= 1 )
                 continue;
 
-            const auto& first_node = *std::min_element(nodes.begin(), nodes.end(), [](const auto& a, const auto& b) {
-                return a->location() < b->location();
-            });
+            auto sorted_nodes = std::vector<Declaration*>(nodes.begin(), nodes.end());
+            // NOLINTNEXTLINE(bugprone-nondeterministic-pointer-iteration-order)
+            std::sort(sorted_nodes.begin(), sorted_nodes.end(),
+                      [](const auto* a, const auto* b) { return a->location() < b->location(); });
+
+            const auto* first_node = *sorted_nodes.begin();
+            for ( std::size_t i = 1; i < sorted_nodes.size(); i++ ) {
+                const auto& node = sorted_nodes[i];
+
+                // Functions can legitimately be overloaded most of the time.
+                if ( auto current_decl = node->tryAs<declaration::Function>();
+                     current_decl && first_node->isA<declaration::Function>() ) {
+                    // Try all previous nodes and see if this is a valid overload for each
+                    for ( std::size_t j = 0; j < i; j++ ) {
+                        if ( auto previous_decl = sorted_nodes[j]->tryAs<declaration::Function>() ) {
+                            auto current_fn_ty = current_decl->function()->ftype();
+                            auto previous_fn_ty = previous_decl->function()->ftype();
+
+                            if ( current_fn_ty->flavor() == type::function::Flavor::Hook &&
+                                 current_fn_ty->flavor() == previous_fn_ty->flavor() )
+                                continue;
+
+                            if ( auto valid = isValidOverload(current_fn_ty, previous_fn_ty); ! valid )
+                                error(fmt("'%s' is not a valid overload: %s; previous definition in %s", id,
+                                          valid.error(), previous_decl->location()),
+                                      node);
+                        }
+                    }
 
-            for ( const auto& node : nodes ) {
-                if ( node->location() == first_node->location() )
                     continue;
-
-                // Functions can legitimately be overloaded.
-                if ( node->isA<declaration::Function>() ) {
-                    if ( first_node->tryAs<declaration::Function>() )
-                        continue;
                 }
 
                 // Modules of the same name can be imported if they come with different scopes.
@@ -176,14 +244,20 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
     }
 
     void operator()(Function* n) final {
+        checkNodeAttributes(n, n->attributes(), "function");
+
         if ( auto attrs = n->attributes() ) {
-            if ( auto prio = attrs->find(hilti::Attribute::Kind::Priority) ) {
-                if ( n->ftype()->flavor() != type::function::Flavor::Hook )
+            auto is_hook = n->ftype()->flavor() == type::function::Flavor::Hook;
+            if ( auto prio = attrs->find(hilti::attribute::kind::Priority) ) {
+                if ( ! is_hook )
                     error("only hooks can have priorities", n);
 
                 else if ( auto x = prio->valueAsInteger(); ! x )
                     error(x.error(), n);
             }
+
+            if ( ! n->body() && ! is_hook && ! attrs->has(hilti::attribute::kind::Cxxname) )
+                error(fmt("function '%s' must have a body or be declared with &cxxname", n->id()), n);
         }
     }
 
@@ -198,6 +272,8 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
     }
 
     void operator()(declaration::Constant* n) final {
+        checkDeclarationType(n, n->type());
+
         if ( n->value()->type()->isWildcard() )
             error("cannot use wildcard type for constants", n);
 
@@ -211,6 +287,8 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
         hilti::visitor::visit(VisitExpressions(this), n);
     }
 
+    void operator()(declaration::Field* n) final { checkDeclarationType(n, n->type()); }
+
     void operator()(declaration::Function* n) final {
         if ( ! operator_::registry().byBuiltinFunctionID(n->id().local()).empty() )
             error("function uses reserved ID", n);
@@ -220,6 +298,8 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
     }
 
     void operator()(declaration::LocalVariable* n) final {
+        checkDeclarationType(n, n->type());
+
         if ( auto t = n->type()->type();
              ! t->isAllocable() && ! t->isA<type::Unknown>() ) // unknown will be reported elsewhere
             error(fmt("type '%s' cannot be used for variable declaration", *n->type()), n);
@@ -260,6 +340,9 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
     }
 
     void operator()(declaration::Parameter* n) final {
+        checkNodeAttributes(n, n->attributes(), n->displayName());
+        checkDeclarationType(n, n->type());
+
         if ( ! n->type()->type()->isA<type::Auto>() ) {
             if ( ! n->type()->type()->isAllocable() && ! n->type()->type()->isA<type::Any>() )
                 error(fmt("type '%s' cannot be used for function parameter", *n->type()), n);
@@ -267,7 +350,7 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
 
         if ( n->type()->isWildcard() ) {
             if ( auto d = n->parent(4)->tryAs<declaration::Function>() ) {
-                if ( ! d->function()->attributes()->has(hilti::Attribute::Kind::Cxxname) )
+                if ( ! d->function()->attributes()->has(hilti::attribute::kind::Cxxname) )
                     error(fmt("parameter '%s' cannot have wildcard type; only allowed with runtime library "
                               "functions declared with &cxxname",
                               n->id()),
@@ -275,7 +358,7 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
             }
 
             if ( auto d = n->parent(4)->tryAs<declaration::Type>() ) {
-                if ( ! d->attributes()->has(hilti::Attribute::Kind::Cxxname) )
+                if ( ! d->attributes()->has(hilti::attribute::kind::Cxxname) )
                     error(fmt("parameter '%s' cannot have wildcard type; only allowed with methods in runtime "
                               "library structs declared with &cxxname",
                               n->id()),
@@ -285,10 +368,7 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
 
         if ( auto attrs = n->attributes() )
             for ( const auto& attr : attrs->attributes() ) {
-                if ( attr->kind() != hilti::Attribute::Kind::RequiresTypeFeature )
-                    error(fmt("invalid attribute '%s' for function parameter", attr->attributeName()), n);
-
-                else {
+                if ( attr->kind() == hilti::attribute::kind::RequiresTypeFeature ) {
                     if ( auto x = attr->valueAsString(); ! x )
                         error(x.error(), n);
                 }
@@ -296,6 +376,8 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
     }
 
     void operator()(declaration::GlobalVariable* n) final {
+        checkDeclarationType(n, n->type());
+
         if ( auto t = n->type()->type();
              ! t->isAllocable() && ! t->isA<type::Unknown>() ) // unknown will be reported elsewhere
             error(fmt("type '%s' cannot be used for variable declaration", *n->type()), n);
@@ -355,7 +437,7 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
     void operator()(ctor::Null* n) final {}
 
     void operator()(ctor::RegExp* n) final {
-        if ( n->attributes()->has(hilti::Attribute::Kind::Anchor) )
+        if ( n->attributes()->has(hilti::attribute::kind::Anchor) )
             // This can end up reporting the same location multiple times,
             // which seems fine. Otherwise we'd need to explicitly track what's
             // reported already.
@@ -418,6 +500,14 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
     }
 
     void operator()(expression::Name* n) final {
+        if ( n->type()->type()->isA<type::Function>() ) {
+            if ( auto* parent = n->parent(); parent && ! parent->tryAs<expression::UnresolvedOperator>() ) {
+                // We only allow function references in the following two contexts.
+                if ( ! parent->isA<operator_::function::Call>() && ! parent->isA<ctor::struct_::Field>() )
+                    error("function must be called", n, node::ErrorPriority::Low);
+            }
+        }
+
         if ( auto decl = n->resolvedDeclaration() ) {
             if ( auto parent = n->parent<Declaration>();
                  decl == parent && ! decl->isA<declaration::Function>() && n->id() != ID("__dd") ) {
@@ -551,7 +641,7 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
             // Customized error message for calls to methods.
             std::vector<std::string> context;
 
-            if ( auto candidates = operator_::registry().byMethodID(n->op1()->as<expression::Member>()->id());
+            if ( const auto& candidates = operator_::registry().byMethodID(n->op1()->as<expression::Member>()->id());
                  ! candidates.empty() ) {
                 // Apply heuristic on op0 to limit the candidates reported.
                 std::vector<std::string> cands;
@@ -664,8 +754,9 @@ struct VisitorPost : visitor::PreOrder, public validator::VisitorMixIn {
         if ( n->isWildcard() )
             return;
 
-        if ( const auto& t = n->dereferencedType(); ! t->type()->isAllocable() )
-            error(fmt("type %s cannot be used inside optional", *t), n);
+        if ( const auto& t = n->dereferencedType();
+             ! t->type()->isAllocable() && ! n->parent(2)->isA_<type::tuple::Element>() )
+            error(fmt("type %s cannot be used inside optional", *t), n, node::ErrorPriority::Low);
     }
 
     void operator()(type::StrongReference* n) final {
