@@ -1,11 +1,10 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
 #include <algorithm>
 #include <string>
 #include <string_view>
-#include <tuple>
 #include <type_traits>
 #include <utility>
 #include <vector>
@@ -19,6 +18,7 @@
 #include <hilti/rt/types/port.h>
 #include <hilti/rt/types/reference.h>
 #include <hilti/rt/types/struct.h>
+#include <hilti/rt/types/tuple.h>
 #include <hilti/rt/util.h>
 
 #include <spicy/rt/filter.h>
@@ -60,7 +60,8 @@ struct ParserPort {
     Direction direction;
 
     // Constructor used by code generator.
-    ParserPort(std::tuple<hilti::rt::Port, Direction> args) : port(std::get<0>(args)), direction(std::get<1>(args)) {}
+    ParserPort(hilti::rt::Tuple<hilti::rt::Port, Direction> args)
+        : port(hilti::rt::tuple::get<0>(args)), direction(hilti::rt::tuple::get<1>(args)) {}
 };
 
 inline std::ostream& operator<<(std::ostream& out, const ParserPort& p) { return out << hilti::rt::to_string(p); }
@@ -314,7 +315,7 @@ inline const auto& parserNames() { return detail::globalState()->parsers_by_name
 
 /**
  * Records an alias name for an already registered parser. The alias
- * name will then be recognized by `Driver::lookupParser()`.
+ * name will then be recognized by `lookupParser()`.
  *
  * @param parser name of the parser to register the alias for
  * @param alias alias name to register the parser under
@@ -322,6 +323,18 @@ inline const auto& parserNames() { return detail::globalState()->parsers_by_name
  */
 hilti::rt::Result<hilti::rt::Nothing> registerParserAlias(const std::string& parser, const std::string& alias);
 
+/**
+ * Retrieves a parser by its name.
+ *
+ * @param name name of the parser to be retrieved, either as shown in the
+ * output of `listParsers()`; or, alternatively, as a string rendering of a
+ * port or MIME type as defined by a unit's properties. If no name is given
+ * and there's only one parser available, that one is taken automatically.
+ *
+ * @return the parser, or an error if it could not be retrieved
+ */
+hilti::rt::Result<const spicy::rt::Parser*> lookupParser(const std::string& name = "");
+
 /**
  * Exception thrown by generated parser code when an parsing failed.
  */
@@ -382,7 +395,8 @@ namespace detail {
  */
 template<typename UnitRef>
 inline void registerParser(::spicy::rt::Parser& p, // NOLINT(google-runtime-references)
-                           std::string linker_scope, UnitRef /* not used, just for template instantiation */) {
+                           std::string linker_scope, UnitRef /* not used, just for template instantiation */,
+                           const hilti::rt::TypeInfo* /* utype */) {
     // Note: This may may be called before spicy::rt::init(), and during
     // hilti::rt::init(). Cannot rely on any library functionality being
     // initialized yet.
