@@ -1,10 +1,11 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <limits>
 #include <utility>
 
 #include <hilti/rt/exception.h>
 #include <hilti/rt/types/bytes.h>
+#include <hilti/rt/types/result.h>
 #include <hilti/rt/types/stream.h>
 
 #include <spicy/rt/configuration.h>
@@ -64,6 +65,33 @@ hilti::rt::Result<hilti::rt::Nothing> spicy::rt::registerParserAlias(const std::
     return hilti::rt::result::Error(hilti::rt::fmt("unknown parser '%s'", parser));
 }
 
+hilti::rt::Result<const spicy::rt::Parser*> spicy::rt::lookupParser(const std::string& name) {
+    const auto& parsers = spicy::rt::parsers();
+
+    if ( parsers.empty() )
+        return hilti::rt::result::Error("no parsers available");
+
+    if ( name.empty() ) {
+        if ( const auto& def = detail::globalState()->default_parser )
+            return *def;
+        else
+            return hilti::rt::result::Error("multiple parsers available, need to select one");
+    }
+
+    const auto& parsers_by_name = detail::globalState()->parsers_by_name;
+
+    if ( auto p = parsers_by_name.find(name); p != parsers_by_name.end() ) {
+        assert(! p->second.empty());
+
+        if ( p->second.size() > 1 )
+            return hilti::rt::result::Error("multiple matching parsers found");
+
+        return p->second.front();
+    }
+    else
+        return hilti::rt::result::Error("no matching parser available");
+}
+
 void detail::printParserState(std::string_view unit_id, const hilti::rt::ValueReference<hilti::rt::Stream>& data,
                               const std::optional<hilti::rt::stream::SafeConstIterator>& begin,
                               const hilti::rt::stream::View& cur, int64_t lahead,
@@ -76,7 +104,7 @@ void detail::printParserState(std::string_view unit_id, const hilti::rt::ValueRe
             if ( i >= end )
                 return std::make_pair(hilti::rt::stream::View(begin, end), "");
 
-            return std::make_pair(hilti::rt::stream::View(begin, i), "...");
+            return std::make_pair(hilti::rt::stream::View(begin, std::move(i)), "...");
         };
 
         auto na = hilti::rt::Stream("n/a");
@@ -217,14 +245,14 @@ std::optional<hilti::rt::stream::SafeConstIterator> detail::unitFind(
     const hilti::rt::stream::SafeConstIterator& begin, const hilti::rt::stream::SafeConstIterator& end,
     const std::optional<hilti::rt::stream::SafeConstIterator>& i, const hilti::rt::Bytes& needle,
     hilti::rt::stream::Direction d) {
-    std::tuple<bool, hilti::rt::stream::SafeConstIterator> v;
+    hilti::rt::Tuple<bool, hilti::rt::stream::SafeConstIterator> v;
     if ( i )
         v = hilti::rt::stream::View(begin, end).find(needle, *i, d);
     else
         v = hilti::rt::stream::View(begin, end).find(needle, d);
 
-    if ( std::get<0>(v) )
-        return std::get<1>(v);
+    if ( hilti::rt::tuple::get<0>(v) )
+        return hilti::rt::tuple::get<1>(v);
     else
         return {};
 }
