@@ -1,10 +1,9 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <doctest/doctest.h>
 
 #include <memory>
 #include <optional>
-#include <utility>
 #include <vector>
 
 #include <hilti/rt/extension-points.h>
@@ -48,11 +47,13 @@ TEST_CASE("Direction") {
 }
 
 TEST_CASE("ParserPort") {
-    CHECK_EQ(to_string(ParserPort({Port(80, Protocol::TCP), Direction::Originator})), "80/tcp (originator direction)");
-    CHECK_EQ(to_string(ParserPort({Port(80, Protocol::TCP), Direction::Both})), "80/tcp");
+    CHECK_EQ(to_string(ParserPort(std::make_tuple(Port(80, Protocol::TCP), Direction::Originator))),
+             "80/tcp (originator direction)");
+    CHECK_EQ(to_string(ParserPort(std::make_tuple(Port(80, Protocol::TCP), Direction::Both))), "80/tcp");
 
-    CHECK_EQ(fmt("%s", ParserPort({Port(80, Protocol::TCP), Direction::Originator})), "80/tcp (originator direction)");
-    CHECK_EQ(fmt("%s", ParserPort({Port(80, Protocol::TCP), Direction::Both})), "80/tcp");
+    CHECK_EQ(fmt("%s", ParserPort(std::make_tuple(Port(80, Protocol::TCP), Direction::Originator))),
+             "80/tcp (originator direction)");
+    CHECK_EQ(fmt("%s", ParserPort(std::make_tuple(Port(80, Protocol::TCP), Direction::Both))), "80/tcp");
 }
 
 TEST_CASE("atEod") {
@@ -145,7 +146,7 @@ TEST_CASE("registerParser") {
         REQUIRE_FALSE(parser.__hook_skipped);
         REQUIRE_FALSE(parser.__hook_undelivered);
 
-        detail::registerParser(parser, "xyz", UnitRef<int>());
+        detail::registerParser(parser, "xyz", UnitRef<int>(), nullptr);
 
         REQUIRE_EQ(detail::globalState()->parsers.size(), 1U);
         CHECK_EQ(detail::globalState()->parsers.at(0), &parser);
@@ -170,7 +171,7 @@ TEST_CASE("registerParser") {
         REQUIRE_FALSE(parser.__hook_skipped);
         REQUIRE_FALSE(parser.__hook_undelivered);
 
-        detail::registerParser(parser, "xyz", UnitRef<UnitWithSinkSupport>());
+        detail::registerParser(parser, "xyz", UnitRef<UnitWithSinkSupport>(), nullptr);
 
         REQUIRE_EQ(detail::globalState()->parsers.size(), 1U);
         CHECK_EQ(detail::globalState()->parsers.at(0), &parser);
@@ -190,7 +191,7 @@ TEST_CASE("registerParser") {
         Parser parser;
         parser.mime_types = {MIMEType("foo/bar"), MIMEType("foo/*")};
 
-        detail::registerParser(parser, "xyz", UnitRef<UnitWithSinkSupport>());
+        detail::registerParser(parser, "xyz", UnitRef<UnitWithSinkSupport>(), nullptr);
 
         REQUIRE_EQ(detail::globalState()->parsers.size(), 1U);
         CHECK_EQ(detail::globalState()->parsers.at(0), &parser);
@@ -204,7 +205,7 @@ TEST_CASE("registerParserAlias") {
     Parser parser;
     parser.name = "parser";
     parser.is_public = true;
-    detail::registerParser(parser, "xyz", UnitRef<int>());
+    detail::registerParser(parser, "xyz", UnitRef<int>(), nullptr);
     detail::__global_state->runtime_is_initialized = false;
     init(); // populates the alias table
 
