@@ -1,9 +1,7 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
-#include <memory>
-#include <string>
 #include <utility>
 
 #include <hilti/ast/attribute.h>
@@ -106,7 +104,7 @@ public:
                                   {"forwarding", _is_forwarding},
                                   {"index", _index},
                                   {"skip", _is_skip}};
-        return unit::Item::properties() + p;
+        return unit::Item::properties() + std::move(p);
     }
 
     static auto create(ASTContext* ctx, const ID& id, QualifiedType* type, bool skip, Expressions args,
