@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
@@ -581,6 +581,9 @@ public:
     QualifiedType* lookAheadType() const;
     hilti::Expression* featureConstant(const type::Unit* unit, std::string_view feature);
 
+    /** Adds a temporary to store an attribute's expression, if necessary. */
+    Expression* evaluateAttributeExpression(const hilti::Attribute* attr, const std::string& prefix);
+
     /*
      * Filters a set of field attributes to remove those that are handled
      * generically by the field parsing machinery that the parser builder sets
