@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <utility>
 
@@ -6,6 +6,7 @@
 #include <hilti/ast/types/struct.h>
 #include <hilti/base/logger.h>
 
+#include <spicy/ast/attribute.h>
 #include <spicy/ast/builder/builder.h>
 #include <spicy/ast/types/unit-items/field.h>
 #include <spicy/ast/visitor.h>
@@ -31,7 +32,7 @@ struct TypeParser {
 
     Expression* buildParser(UnqualifiedType* t);
 
-    auto state() { return pb->state(); }
+    const auto& state() { return pb->state(); }
     auto builder() { return pb->builder(); }
     auto context() { return pb->context(); }
     auto pushBuilder(std::shared_ptr<Builder> b) { return pb->pushBuilder(std::move(b)); }
@@ -71,7 +72,7 @@ struct TypeParser {
             auto result = dst ? dst : builder()->addTmp("result", builder()->typeResult(qt));
 
             auto true_ = builder()->addIf(has_data);
-            pushBuilder(true_);
+            pushBuilder(std::move(true_));
             auto unpacked = builder()->deref(builder()->unpack(qt, unpack_args));
             builder()->addAssign(builder()->tuple({result, state().cur}), unpacked);
             popBuilder();
@@ -85,10 +86,10 @@ struct TypeParser {
     Expression* fieldByteOrder() {
         Expression* byte_order = nullptr;
 
-        if ( const auto& a = meta.field()->attributes()->find(hilti::Attribute::Kind::ByteOrder) )
+        if ( const auto& a = meta.field()->attributes()->find(attribute::kind::ByteOrder) )
             byte_order = *a->valueAsExpression();
 
-        else if ( const auto& a = state().unit->attributes()->find(hilti::Attribute::Kind::ByteOrder) )
+        else if ( const auto& a = state().unit->attributes()->find(attribute::kind::ByteOrder) )
             byte_order = *a->valueAsExpression();
 
         else if ( const auto& p = state().unit->propertyItem("%byte-order") )
@@ -122,8 +123,8 @@ struct Visitor : public visitor::PreOrder {
     void operator()(hilti::type::Address* n) final {
         switch ( tp->mode ) {
             case TypesMode::Default: {
-                auto v4 = tp->meta.field()->attributes()->find(hilti::Attribute::Kind::IPv4);
-                auto v6 = tp->meta.field()->attributes()->find(hilti::Attribute::Kind::IPv6);
+                auto v4 = tp->meta.field()->attributes()->find(attribute::kind::IPv4);
+                auto v6 = tp->meta.field()->attributes()->find(attribute::kind::IPv6);
                 (void)v6;
                 assert(! (v4 && v6));
 
@@ -159,7 +160,7 @@ struct Visitor : public visitor::PreOrder {
                 Expression* bitorder = builder()->id("hilti::BitOrder::LSB0");
 
                 if ( auto attrs = n->attributes() ) {
-                    if ( auto a = attrs->find(hilti::Attribute::Kind::BitOrder) )
+                    if ( auto a = attrs->find(attribute::kind::BitOrder) )
                         bitorder = *a->valueAsExpression();
                 }
 
@@ -169,7 +170,7 @@ struct Visitor : public visitor::PreOrder {
 
                 if ( pb()->options().debug ) {
                     auto have_value = builder()->addIf(builder()->hasMember(target, "__value__"));
-                    pushBuilder(have_value, [&]() {
+                    pushBuilder(std::move(have_value), [&]() {
                         // Print all the bit ranges individually so that we can include
                         // their IDs, which the standard tuple output wouldn't show.
                         builder()->addDebugMsg("spicy", fmt("%s = %%s", tp->meta.field()->id()),
@@ -199,7 +200,7 @@ struct Visitor : public visitor::PreOrder {
     void operator()(hilti::type::Real* n) final {
         switch ( tp->mode ) {
             case TypesMode::Default: {
-                auto type = tp->meta.field()->attributes()->find(hilti::Attribute::Kind::Type);
+                auto type = tp->meta.field()->attributes()->find(attribute::kind::Type);
                 assert(type);
                 result = tp->performUnpack(tp->destination(n), builder()->typeReal(), 4,
                                            {state().cur, *type->valueAsExpression(), tp->fieldByteOrder()}, n->meta(),
@@ -271,17 +272,17 @@ struct Visitor : public visitor::PreOrder {
 
     void operator()(hilti::type::Bytes* n) final {
         auto attrs = tp->meta.field()->attributes();
-        auto chunked_attr = attrs->find(hilti::Attribute::Kind::Chunked);
-        auto eod_attr = attrs->find(hilti::Attribute::Kind::Eod);
-        auto size_attr = attrs->find(hilti::Attribute::Kind::Size);
-        auto until_attr = attrs->find(hilti::Attribute::Kind::Until);
-        auto until_including_attr = attrs->find(hilti::Attribute::Kind::UntilIncluding);
+        auto chunked_attr = attrs->find(attribute::kind::Chunked);
+        auto eod_attr = attrs->find(attribute::kind::Eod);
+        auto size_attr = attrs->find(attribute::kind::Size);
+        auto until_attr = attrs->find(attribute::kind::Until);
+        auto until_including_attr = attrs->find(attribute::kind::UntilIncluding);
 
         bool to_eod = (eod_attr != nullptr); // parse to end of input data
         bool parse_attr = false;             // do we have a &parse-* attribute
 
-        if ( (tp->meta.field()->attributes()->find(hilti::Attribute::Kind::ParseFrom) ||
-              tp->meta.field()->attributes()->find(hilti::Attribute::Kind::ParseAt)) &&
+        if ( (tp->meta.field()->attributes()->find(attribute::kind::ParseFrom) ||
+              tp->meta.field()->attributes()->find(attribute::kind::ParseAt)) &&
              ! (until_attr || until_including_attr) )
             parse_attr = true;
 
@@ -303,11 +304,11 @@ struct Visitor : public visitor::PreOrder {
 
                     if ( chunked_attr ) {
                         auto loop = builder()->addWhile(builder()->bool_(true));
-                        pushBuilder(loop, [&]() {
+                        pushBuilder(std::move(loop), [&]() {
                             builder()->addLocal("more_data", pb()->waitForInputOrEod(builder()->integer(1)));
 
                             auto have_data = builder()->addIf(builder()->size(state().cur));
-                            pushBuilder(have_data, [&]() {
+                            pushBuilder(std::move(have_data), [&]() {
                                 builder()->addAssign(target, state().cur);
                                 pb()->advanceInput(builder()->size(state().cur));
 
@@ -359,7 +360,7 @@ struct Visitor : public visitor::PreOrder {
 
                     builder()->addAssign(target, builder()->bytes(""));
                     auto body = builder()->addWhile(builder()->bool_(true));
-                    pushBuilder(body, [&]() {
+                    pushBuilder(std::move(body), [&]() {
                         // Helper to add a new chunk of data to the field's value,
                         // behaving slightly different depending on whether we have
                         // &chunked or not.
@@ -384,20 +385,20 @@ struct Visitor : public visitor::PreOrder {
                         auto it_id = ID("it");
                         auto found = builder()->id(found_id);
                         auto it = builder()->id(it_id);
-                        builder()->addLocal(found_id,
+                        builder()->addLocal(std::move(found_id),
                                             builder()->qualifiedType(builder()->typeBool(), hilti::Constness::Mutable));
-                        builder()->addLocal(it_id, builder()->qualifiedType(builder()->typeStreamIterator(),
-                                                                            hilti::Constness::Mutable));
+                        builder()->addLocal(std::move(it_id), builder()->qualifiedType(builder()->typeStreamIterator(),
+                                                                                       hilti::Constness::Mutable));
                         builder()->addAssign(builder()->tuple({found, it}), find);
 
                         Expression* match = builder()->memberCall(state().cur, "sub", {it});
 
                         auto non_empty_match = builder()->addIf(builder()->size(match));
-                        pushBuilder(non_empty_match, [&]() { add_match_data(target, match); });
+                        pushBuilder(std::move(non_empty_match), [&]() { add_match_data(target, match); });
 
                         auto [found_branch, not_found_branch] = builder()->addIfElse(found);
 
-                        pushBuilder(found_branch, [&]() {
+                        pushBuilder(std::move(found_branch), [&]() {
                             auto new_it = builder()->sum(it, until_bytes_size_var);
 
                             if ( until_including_attr )
@@ -407,7 +408,7 @@ struct Visitor : public visitor::PreOrder {
                             builder()->addBreak();
                         });
 
-                        pushBuilder(not_found_branch, [&]() { pb()->advanceInput(it); });
+                        pushBuilder(std::move(not_found_branch), [&]() { pb()->advanceInput(it); });
                     });
 
                     result = target;
@@ -422,7 +423,7 @@ struct Visitor : public visitor::PreOrder {
             case TypesMode::Optimize: {
                 auto parse_attrs = ParserBuilder::removeGenericParseAttributes(attrs);
                 if ( size_attr && parse_attrs.size() == 0 ) {
-                    auto length = *size_attr->valueAsExpression();
+                    auto length = pb()->evaluateAttributeExpression(size_attr, "size");
                     auto eod_ok = builder()->bool_(eod_attr ? true : false);
                     auto value =
                         builder()->call("spicy_rt::extractBytes", {state().data, state().cur, length, eod_ok,
