@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #include <hilti/ast/ast-context.h>
 #include <hilti/base/logger.h>
@@ -69,9 +69,8 @@ struct VisitorScopeBuilder : visitor::PostOrder {
     }
 
     void operator()(hilti::Attribute* n) final {
-        if ( hilti::Attribute::isOneOf(n->kind(),
-                                       {hilti::Attribute::Kind::Until, hilti::Attribute::Kind::UntilIncluding,
-                                        hilti::Attribute::Kind::While}) ) {
+        if ( hilti::attribute::isOneOf(n->kind(), {attribute::kind::Until, attribute::kind::UntilIncluding,
+                                                   attribute::kind::While}) ) {
             auto f = n->parent<type::unit::item::Field>();
             if ( ! (f && f->isContainer()) )
                 return;
