@@ -1,22 +1,24 @@
 #! /usr/bin/env python3
 
-import sys
 import os
 
 cflags = os.environ.get("CFLAGS", "")
 os.environ["CFLAGS"] = cflags + " -fno-strict-aliasing"
 
-from setuptools import setup
 from distutils.core import Extension
 
+from setuptools import setup
+
 setup(
-    version="0.37.dev5", # Filled in automatically.
-    py_modules=['SubnetTree'],
+    version="0.37.dev22",  # Filled in automatically.
+    py_modules=["SubnetTree"],
     url="https://github.com/zeek/pysubnettree",
-    ext_modules = [
-        Extension("_SubnetTree",
+    ext_modules=[
+        Extension(
+            "_SubnetTree",
             sources=["SubnetTree.cc", "patricia.c", "SubnetTree_wrap.cc"],
             depends=["include/SubnetTree.h", "include/patricia.h"],
-            include_dirs=["include/"])
-        ],
+            include_dirs=["include/"],
+        )
+    ],
 )
