@@ -22,7 +22,7 @@ DebugLogger::Stream DebugLogger::streams[NUM_DBGS] =
      {"tm", 0, false},        {"logging", 0, false},    {"input", 0, false},           {"threading", 0, false},
      {"plugins", 0, false},   {"zeekygen", 0, false},   {"pktio", 0, false},           {"broker", 0, false},
      {"scripts", 0, false},   {"supervisor", 0, false}, {"hashkey", 0, false},         {"spicy", 0, false},
-     {"cluster", 0, false}};
+     {"cluster", 0, false},   {"storage", 0, false}};
 
 DebugLogger::~DebugLogger() {
     if ( file && file != stderr )
@@ -122,7 +122,7 @@ void DebugLogger::EnableStreams(const char* s) {
         for ( i = 0; i < NUM_DBGS; ++i ) {
             if ( ltok == streams[i].prefix ) {
                 streams[i].enabled = true;
-                enabled_streams.insert(ltok);
+                enabled_streams.insert(std::move(ltok));
                 goto next;
             }
         }
