@@ -36,6 +36,7 @@ extern double non_analyzed_lifetime;
 extern double tcp_inactivity_timeout;
 extern double udp_inactivity_timeout;
 extern double icmp_inactivity_timeout;
+extern double unknown_ip_inactivity_timeout;
 
 extern int tcp_storm_thresh;
 extern double tcp_storm_interarrival_thresh;
@@ -85,8 +86,6 @@ extern int dpd_match_only_beginning;
 extern int dpd_late_match_stop;
 extern int dpd_ignore_ports;
 
-extern int check_for_unused_event_handlers;
-
 extern int record_all_packets;
 
 extern zeek_uint_t bits_per_uid;
@@ -107,4 +106,6 @@ extern void init_builtin_types();
 #include "packet_analysis.bif.netvar_h"
 #include "reporter.bif.netvar_h"
 #include "supervisor.bif.netvar_h"
+#include "telemetry_consts.bif.netvar_h"
+#include "telemetry_types.bif.netvar_h"
 #include "types.bif.netvar_h"
