@@ -9,12 +9,12 @@
 #include <syslog.h>
 #include <unistd.h>
 
-#include "zeek/3rdparty/doctest.h"
 #include "zeek/Conn.h"
 #include "zeek/Desc.h"
 #include "zeek/Event.h"
 #include "zeek/EventHandler.h"
 #include "zeek/Expr.h"
+#include "zeek/Frame.h"
 #include "zeek/ID.h"
 #include "zeek/NetVar.h"
 #include "zeek/RunState.h"
@@ -24,6 +24,8 @@
 #include "zeek/plugin/Manager.h"
 #include "zeek/plugin/Plugin.h"
 
+#include "zeek/3rdparty/doctest.h"
+
 #ifdef SYSLOG_INT
 extern "C" {
 int openlog(const char* ident, int logopt, int facility);
@@ -203,6 +205,14 @@ void Reporter::CPPRuntimeError(const char* fmt, ...) {
     throw InterpreterException();
 }
 
+void Reporter::CPPRuntimeWarning(const char* fmt, ...) {
+    va_list ap;
+    va_start(ap, fmt);
+    FILE* out = EmitToStderr(warnings_to_stderr) ? stderr : nullptr;
+    DoLog("runtime warning in compiled code", reporter_error, out, nullptr, nullptr, true, true, "", fmt, ap);
+    va_end(ap);
+}
+
 void Reporter::InternalError(const char* fmt, ...) {
     va_list ap;
     va_start(ap, fmt);
@@ -640,4 +650,10 @@ void Reporter::DoLog(const char* prefix, EventHandlerPtr event, FILE* out, Conne
 
 bool Reporter::EmitToStderr(bool flag) { return flag || ! run_state::detail::zeek_init_done; }
 
+ScriptLocationScope::ScriptLocationScope(const zeek::detail::Frame* frame) {
+    zeek::reporter->PushLocation(frame->GetCallLocation());
+}
+
+ScriptLocationScope::~ScriptLocationScope() { zeek::reporter->PopLocation(); }
+
 } // namespace zeek
