@@ -58,8 +58,9 @@ enum TimerType : uint8_t {
     TIMER_UNKNOWN_PROTOCOL_EXPIRE,
     TIMER_LOG_DELAY_EXPIRE,
     TIMER_LOG_FLUSH_WRITE_BUFFER,
+    TIMER_STORAGE_EXPIRE,
 };
-constexpr int NUM_TIMER_TYPES = int(TIMER_LOG_FLUSH_WRITE_BUFFER) + 1;
+constexpr int NUM_TIMER_TYPES = int(TIMER_STORAGE_EXPIRE) + 1;
 
 extern const char* timer_type_to_string(TimerType type);
 
