@@ -62,6 +62,7 @@ union ZVal {
     ZVal(const TypePtr& t);
 
     // Construct directly.
+    ZVal(bool v) { int_val = v; }
     ZVal(zeek_int_t v) { int_val = v; }
     ZVal(zeek_uint_t v) { uint_val = v; }
     ZVal(double v) { double_val = v; }
@@ -160,7 +161,6 @@ union ZVal {
 private:
     friend class RecordVal;
     friend class VectorVal;
-    friend class zeek::detail::ZBody;
 
     // Used for bool, int, enum.
     zeek_int_t int_val;
