@@ -21,12 +21,19 @@ namespace zeek::analyzer::dns {
 
 namespace detail {
 
+// Used for checking whether the connection being parsed comes from NetBIOS,
+// since it's similar to DNS but does some things differently.
+constexpr int NETBIOS_PORT = 137;
+
 DNS_Interpreter::DNS_Interpreter(analyzer::Analyzer* arg_analyzer) {
     analyzer = arg_analyzer;
     first_message = true;
+    is_netbios =
+        ntohs(analyzer->Conn()->OrigPort()) == NETBIOS_PORT || ntohs(analyzer->Conn()->RespPort()) == NETBIOS_PORT;
 }
 
 void DNS_Interpreter::ParseMessage(const u_char* data, int len, int is_query) {
+    // Every packet for every opcode starts with same size header.
     int hdr_len = sizeof(detail::DNS_RawMsgHdr);
 
     if ( len < hdr_len ) {
@@ -34,7 +41,21 @@ void DNS_Interpreter::ParseMessage(const u_char* data, int len, int is_query) {
         return;
     }
 
-    detail::DNS_MsgInfo msg((detail::DNS_RawMsgHdr*)data, is_query);
+    // The flags section may be different between the different opcodes, but the
+    // opcode is always in the same location. Parse out just that part of it here
+    // even though it will probably be reparsed later.
+    auto* hdr = (detail::DNS_RawMsgHdr*)data;
+    unsigned short flags = ntohs(hdr->flags);
+    int opcode = (flags & 0x7800) >> 11;
+
+    // NetBIOS registration and release messages look like regular DNS requests, so parse them as such
+    if ( opcode != DNS_OP_QUERY && ! is_netbios ) {
+        analyzer->Weird("DNS_unknown_opcode", util::fmt("%d", opcode));
+        analyzer->Conn()->CheckHistory(zeek::session::detail::HIST_UNKNOWN_PKT, 'X');
+        return;
+    }
+
+    detail::DNS_MsgInfo msg(hdr, is_query);
 
     if ( first_message && msg.QR && is_query == 1 ) {
         is_query = msg.is_query = 0;
@@ -242,7 +263,7 @@ bool DNS_Interpreter::ParseAnswer(detail::DNS_MsgInfo* msg, const u_char*& data,
         case detail::TYPE_NBS: status = ParseRR_NBS(msg, data, len, rdlength, msg_start); break;
 
         case detail::TYPE_SRV:
-            if ( ntohs(analyzer->Conn()->RespPort()) == 137 ) {
+            if ( ntohs(analyzer->Conn()->RespPort()) == NETBIOS_PORT ) {
                 // This is an NBSTAT (NetBIOS NODE STATUS) record.
                 // The SRV RFC reused the value that was already being
                 // used for this.
@@ -256,6 +277,8 @@ bool DNS_Interpreter::ParseAnswer(detail::DNS_MsgInfo* msg, const u_char*& data,
 
         case detail::TYPE_EDNS: status = ParseRR_EDNS(msg, data, len, rdlength, msg_start); break;
 
+        case detail::TYPE_TKEY: status = ParseRR_TKEY(msg, data, len, rdlength, msg_start); break;
+
         case detail::TYPE_TSIG: status = ParseRR_TSIG(msg, data, len, rdlength, msg_start); break;
 
         case detail::TYPE_RRSIG: status = ParseRR_RRSIG(msg, data, len, rdlength, msg_start); break;
@@ -383,7 +406,7 @@ bool DNS_Interpreter::ExtractLabel(const u_char*& data, int& len, u_char*& name,
 
     if ( label_len > 63 &&
          // NetBIOS name service look ups can use longer labels.
-         ntohs(analyzer->Conn()->RespPort()) != 137 ) {
+         ntohs(analyzer->Conn()->RespPort()) != NETBIOS_PORT ) {
         analyzer->Weird("DNS_label_too_long");
         return false;
     }
@@ -824,6 +847,48 @@ bool DNS_Interpreter::ParseRR_TSIG(detail::DNS_MsgInfo* msg, const u_char*& data
     return true;
 }
 
+bool DNS_Interpreter::ParseRR_TKEY(detail::DNS_MsgInfo* msg, const u_char*& data, int& len, int rdlength,
+                                   const u_char* msg_start) {
+    if ( ! dns_TKEY || msg->skip_event ) {
+        data += rdlength;
+        len -= rdlength;
+        return true;
+    }
+
+    if ( len < 16 )
+        return false;
+
+    const u_char* data_start = data;
+    u_char alg_name[513];
+    int alg_name_len = sizeof(alg_name) - 1;
+
+    u_char* alg_name_end = ExtractName(data, len, alg_name, alg_name_len, msg_start);
+
+    if ( ! alg_name_end )
+        return false;
+
+    uint32_t inception = ExtractLong(data, len);
+    uint32_t expiration = ExtractLong(data, len);
+    uint16_t mode = ExtractShort(data, len);
+    uint16_t error = ExtractShort(data, len);
+    String* key_data;
+    ExtractOctets(data, len, dns_TKEY ? &key_data : nullptr);
+    ExtractOctets(data, len, nullptr); // Other data
+
+    if ( dns_TKEY ) {
+        detail::TKEY_DATA tkey;
+        tkey.alg_name = new String(alg_name, int(alg_name_end - alg_name), true);
+        tkey.inception = inception;
+        tkey.expiration = expiration;
+        tkey.mode = mode;
+        tkey.error = error;
+        tkey.key = key_data;
+        analyzer->EnqueueConnEvent(dns_TKEY, analyzer->ConnVal(), msg->BuildHdrVal(), msg->BuildTKEY_Val(&tkey));
+    }
+
+    return true;
+}
+
 bool DNS_Interpreter::ParseRR_RRSIG(detail::DNS_MsgInfo* msg, const u_char*& data, int& len, int rdlength,
                                     const u_char* msg_start) {
     if ( ! dns_RRSIG || msg->skip_event ) {
@@ -1197,6 +1262,14 @@ bool DNS_Interpreter::ParseRR_BINDS(detail::DNS_MsgInfo* msg, const u_char*& dat
 
     String* completeflag = ExtractStream(data, len, rdlength - 4);
 
+    // We exposed the complete flag as a string to script land previously,
+    // but there should only ever be a single byte, so raise a weird if
+    // it is longer than that.
+    //
+    // https://bind9.readthedocs.io/en/latest/chapter5.html#monitoring-with-private-type-records
+    if ( completeflag->Len() > 1 )
+        analyzer->Weird("DNS_BINDS_complete_flag_length", util::fmt("%d", completeflag->Len()));
+
     if ( dns_BINDS ) {
         detail::BINDS_DATA binds;
         binds.algorithm = algo;
@@ -1660,6 +1733,23 @@ RecordValPtr DNS_MsgInfo::BuildEDNS_COOKIE_Val(struct EDNS_COOKIE* opt) {
     return r;
 }
 
+RecordValPtr DNS_MsgInfo::BuildTKEY_Val(struct TKEY_DATA* tkey) {
+    static auto dns_tkey = id::find_type<RecordType>("dns_tkey");
+    auto r = make_intrusive<RecordVal>(dns_tkey);
+
+    r->Assign(0, query_name);
+    r->Assign(1, answer_type);
+    r->Assign(2, tkey->alg_name);
+    r->AssignTime(3, static_cast<double>(tkey->inception));
+    r->AssignTime(4, static_cast<double>(tkey->expiration));
+    r->Assign(5, static_cast<uint16_t>(tkey->mode));
+    r->Assign(6, static_cast<uint16_t>(tkey->error));
+    r->Assign(7, tkey->key);
+    r->Assign(8, is_query);
+
+    return r;
+}
+
 RecordValPtr DNS_MsgInfo::BuildTSIG_Val(struct TSIG_DATA* tsig) {
     static auto dns_tsig_additional = id::find_type<RecordType>("dns_tsig_additional");
     auto r = make_intrusive<RecordVal>(dns_tsig_additional);
@@ -1773,8 +1863,9 @@ RecordValPtr DNS_MsgInfo::BuildBINDS_Val(BINDS_DATA* binds) {
     r->Assign(2, binds->algorithm);
     r->Assign(3, binds->key_id);
     r->Assign(4, binds->removal_flag);
-    r->Assign(5, binds->complete_flag);
+    r->Assign(5, binds->complete_flag); // Remove in v8.1: Move field 7 here. Drop String* usage.
     r->Assign(6, is_query);
+    r->Assign(7, binds->complete_flag->Len() > 0 ? binds->complete_flag->Bytes()[0] : 0);
 
     return r;
 }
