@@ -264,6 +264,15 @@ bool Manager::CreateStream(Stream* info, RecordVal* description) {
     return true;
 }
 
+// Return true if v is a TypeVal that contains a record type, else false.
+static bool is_record_type_val(const zeek::ValPtr& v) {
+    const auto& t = v->GetType();
+    return t->Tag() == TYPE_TYPE && t->AsTypeType()->GetType()->Tag() == TYPE_RECORD;
+}
+
+// Return true if v contains a FuncVal, else false.
+static bool is_func_val(const zeek::ValPtr& v) { return v->GetType()->Tag() == TYPE_FUNC; }
+
 bool Manager::CreateEventStream(RecordVal* fval) {
     RecordType* rtype = fval->GetType()->AsRecordType();
     if ( ! same_type(rtype, BifType::Record::Input::EventDescription, false) ) {
@@ -274,11 +283,21 @@ bool Manager::CreateEventStream(RecordVal* fval) {
     string stream_name = fval->GetFieldOrDefault("name")->AsString()->CheckString();
 
     auto fields_val = fval->GetFieldOrDefault("fields");
+    if ( ! is_record_type_val(fields_val) ) {
+        reporter->Error("Input stream %s: 'idx' field is not a record type", stream_name.c_str());
+        return false;
+    }
+
     RecordType* fields = fields_val->AsType()->AsTypeType()->GetType()->AsRecordType();
 
     auto want_record = fval->GetFieldOrDefault("want_record");
 
     auto ev_val = fval->GetFieldOrDefault("ev");
+    if ( ev_val && ! is_func_val(ev_val) ) {
+        reporter->Error("Input stream %s: 'ev' field is not an event", stream_name.c_str());
+        return false;
+    }
+
     Func* event = ev_val->AsFunc();
 
     const auto& etype = event->GetType();
@@ -356,6 +375,11 @@ bool Manager::CreateEventStream(RecordVal* fval) {
         assert(false);
 
     auto error_event_val = fval->GetFieldOrDefault("error_ev");
+    if ( error_event_val && ! is_func_val(error_event_val) ) {
+        reporter->Error("Input stream %s: 'error_ev' field is not an event", stream_name.c_str());
+        return false;
+    }
+
     Func* error_event = error_event_val ? error_event_val->AsFunc() : nullptr;
 
     if ( ! CheckErrorEventTypes(stream_name, error_event, false) )
@@ -388,8 +412,8 @@ bool Manager::CreateEventStream(RecordVal* fval) {
 
     stream->num_fields = fieldsV.size();
     stream->fields = fields->Ref()->AsRecordType();
-    stream->event = event_registry->Lookup(event->Name());
-    stream->error_event = error_event ? event_registry->Lookup(error_event->Name()) : nullptr;
+    stream->event = event_registry->Lookup(event->GetName());
+    stream->error_event = error_event ? event_registry->Lookup(error_event->GetName()) : nullptr;
     stream->want_record = (want_record->InternalInt() == 1);
 
     assert(stream->reader);
@@ -414,15 +438,31 @@ bool Manager::CreateTableStream(RecordVal* fval) {
 
     auto pred = fval->GetFieldOrDefault("pred");
     auto idx_val = fval->GetFieldOrDefault("idx");
+    if ( ! is_record_type_val(idx_val) ) {
+        reporter->Error("Input stream %s: 'idx' field is not a record type", stream_name.c_str());
+        return false;
+    }
+
     RecordType* idx = idx_val->AsType()->AsTypeType()->GetType()->AsRecordType();
 
     RecordTypePtr val;
     auto val_val = fval->GetFieldOrDefault("val");
 
-    if ( val_val )
+    if ( val_val ) {
+        if ( ! is_record_type_val(val_val) ) {
+            reporter->Error("Input stream %s: 'val' field is not a record type", stream_name.c_str());
+            return false;
+        }
+
         val = val_val->AsType()->AsTypeType()->GetType<RecordType>();
+    }
 
     auto dst = fval->GetFieldOrDefault("destination");
+    if ( ! dst->GetType()->IsSet() && ! dst->GetType()->IsTable() ) {
+        reporter->Error("Input stream %s: 'destination' field has type %s, expected table or set identifier",
+                        stream_name.c_str(), obj_desc_short(dst->GetType().get()).c_str());
+        return false;
+    }
 
     // check if index fields match table description
     size_t num = idx->NumFields();
@@ -497,6 +537,11 @@ bool Manager::CreateTableStream(RecordVal* fval) {
     }
 
     auto event_val = fval->GetFieldOrDefault("ev");
+    if ( event_val && ! is_func_val(event_val) ) {
+        reporter->Error("Input stream %s: 'ev' field is not an event", stream_name.c_str());
+        return false;
+    }
+
     Func* event = event_val ? event_val->AsFunc() : nullptr;
 
     if ( event ) {
@@ -572,6 +617,11 @@ bool Manager::CreateTableStream(RecordVal* fval) {
     }
 
     auto error_event_val = fval->GetFieldOrDefault("error_ev");
+    if ( error_event_val && ! is_func_val(error_event_val) ) {
+        reporter->Error("Input stream %s: 'error_ev' field is not an event", stream_name.c_str());
+        return false;
+    }
+
     Func* error_event = error_event_val ? error_event_val->AsFunc() : nullptr;
 
     if ( ! CheckErrorEventTypes(stream_name, error_event, true) )
@@ -629,8 +679,8 @@ bool Manager::CreateTableStream(RecordVal* fval) {
     stream->tab = dst.release()->AsTableVal();
     stream->rtype = val.release();
     stream->itype = idx->Ref()->AsRecordType();
-    stream->event = event ? event_registry->Lookup(event->Name()) : nullptr;
-    stream->error_event = error_event ? event_registry->Lookup(error_event->Name()) : nullptr;
+    stream->event = event ? event_registry->Lookup(event->GetName()) : nullptr;
+    stream->error_event = error_event ? event_registry->Lookup(error_event->GetName()) : nullptr;
     stream->currDict = new PDict<InputHash>;
     stream->currDict->SetDeleteFunc(input_hash_delete_func);
     stream->lastDict = new PDict<InputHash>;
@@ -1716,7 +1766,7 @@ RecordVal* Manager::ValueToRecordVal(const Stream* stream, const Value* const* v
             // them and has been warned by reporter.
             // Hence -> assign null to the field, done.
 
-            // Better check that it really is optional. Uou never know.
+            // Better check that it really is optional. You never know.
             assert(request_type->FieldDecl(i)->GetAttr(zeek::detail::ATTR_OPTIONAL));
         }
         else if ( ! vals[*position]->present && ! request_type->FieldDecl(i)->GetAttr(zeek::detail::ATTR_OPTIONAL) ) {
