@@ -1,7 +1,11 @@
+// See the file "COPYING" in the main distribution directory for copyright.
+
 #include "zeek/logging/WriterFrontend.h"
 
 #include "zeek/RunState.h"
+#include "zeek/Span.h"
 #include "zeek/broker/Manager.h"
+#include "zeek/cluster/Backend.h"
 #include "zeek/logging/Manager.h"
 #include "zeek/logging/WriterBackend.h"
 #include "zeek/threading/SerialTypes.h"
@@ -50,18 +54,16 @@ private:
 
 class WriteMessage final : public threading::InputMessage<WriterBackend> {
 public:
-    WriteMessage(WriterBackend* backend, int num_fields, int num_writes, Value*** vals)
+    WriteMessage(WriterBackend* backend, int num_fields, std::vector<detail::LogRecord>&& records)
         : threading::InputMessage<WriterBackend>("Write", backend),
           num_fields(num_fields),
-          num_writes(num_writes),
-          vals(vals) {}
+          records(std::move(records)) {}
 
-    bool Process() override { return Object()->Write(num_fields, num_writes, vals); }
+    bool Process() override { return Object()->Write(num_fields, zeek::Span{records}); }
 
 private:
     int num_fields;
-    int num_writes;
-    Value*** vals;
+    std::vector<detail::LogRecord> records;
 };
 
 class SetBufMessage final : public threading::InputMessage<WriterBackend> {
@@ -89,18 +91,18 @@ private:
 // Frontend methods.
 
 WriterFrontend::WriterFrontend(const WriterBackend::WriterInfo& arg_info, EnumVal* arg_stream, EnumVal* arg_writer,
-                               bool arg_local, bool arg_remote) {
-    stream = arg_stream;
-    writer = arg_writer;
-    Ref(stream);
-    Ref(writer);
+                               bool arg_local, bool arg_remote)
+    : write_buffer(detail::WriteBuffer(BifConst::Log::write_buffer_size)) {
+    // The header's fields are initialized in Init()
+    header = detail::LogWriteHeader{{zeek::NewRef{}, arg_stream},
+                                    {zeek::NewRef{}, arg_writer},
+                                    arg_info.filter_name,
+                                    arg_info.path};
 
     disabled = initialized = false;
     buf = true;
     local = arg_local;
     remote = arg_remote;
-    write_buffer = nullptr;
-    write_buffer_pos = 0;
     info = new WriterBackend::WriterInfo(arg_info);
 
     num_fields = 0;
@@ -110,7 +112,7 @@ WriterFrontend::WriterFrontend(const WriterBackend::WriterInfo& arg_info, EnumVa
     name = util::copy_string(util::fmt("%s/%s", arg_info.path, w));
 
     if ( local ) {
-        backend = log_mgr->CreateBackend(this, writer);
+        backend = log_mgr->CreateBackend(this, header.writer_id.get());
 
         if ( backend )
             backend->Start();
@@ -126,15 +128,12 @@ WriterFrontend::~WriterFrontend() {
 
     delete[] fields;
 
-    Unref(stream);
-    Unref(writer);
     delete info;
     delete[] name;
 }
 
 void WriterFrontend::Stop() {
     if ( disabled ) {
-        CleanupWriteBuffer();
         return;
     }
 
@@ -169,61 +168,76 @@ void WriterFrontend::Init(int arg_num_fields, const Field* const* arg_fields) {
     }
 
     if ( remote ) {
-        broker_mgr->PublishLogCreate(stream, writer, *info, arg_num_fields, arg_fields);
+        broker_mgr->PublishLogCreate(header.stream_id.get(), header.writer_id.get(), *info, arg_num_fields, arg_fields);
     }
+
+    header.fields.reserve(arg_num_fields);
+    for ( int i = 0; i < arg_num_fields; i++ )
+        header.fields.emplace_back(*arg_fields[i]);
 }
 
-void WriterFrontend::Write(int arg_num_fields, Value** vals) {
-    if ( disabled ) {
-        DeleteVals(arg_num_fields, vals);
+void WriterFrontend::Write(detail::LogRecord&& arg_vals) {
+    std::vector<threading::Value> vals = std::move(arg_vals);
+
+    if ( disabled )
+        return;
+
+    if ( vals.size() != static_cast<size_t>(num_fields) ) {
+        reporter->Warning("WriterFrontend %s expected %d fields in write, got %zu. Skipping line.", name, num_fields,
+                          vals.size());
         return;
     }
 
-    if ( arg_num_fields != num_fields ) {
-        reporter->Warning("WriterFrontend %s expected %d fields in write, got %d. Skipping line.", name, num_fields,
-                          arg_num_fields);
-        DeleteVals(arg_num_fields, vals);
-        return;
-    }
+    // If remote logging is enabled *and* broker is used as cluster backend,
+    // push the single log record directly to broker_mgr, it uses its own
+    // buffering logic currently.
+    //
+    // Other cluster backends leverage the write buffering logic in the
+    // WriterFrontend. See FlushWriteBuffer().
+    const bool broker_is_cluster_backend = zeek::cluster::backend == zeek::broker_mgr;
 
     if ( remote ) {
-        broker_mgr->PublishLogWrite(stream, writer, info->path, num_fields, vals);
-    }
+        if ( broker_is_cluster_backend ) {
+            zeek::broker_mgr->PublishLogWrite(header.stream_id.get(), header.writer_id.get(), info->path, vals);
 
-    if ( ! backend ) {
-        DeleteVals(arg_num_fields, vals);
+            if ( ! backend ) // nothing left do do if we do not log locally
+                return;
+        }
+    }
+    else if ( ! backend ) {
+        assert(! remote);
+        // Not remote and no backend, we're done.
         return;
     }
 
-    if ( ! write_buffer ) {
-        // Need new buffer.
-        write_buffer = new Value**[WRITER_BUFFER_SIZE];
-        write_buffer_pos = 0;
-    }
+    // Either non-broker remote or local logging.
+    assert(backend || (remote && ! broker_is_cluster_backend));
 
-    write_buffer[write_buffer_pos++] = vals;
+    write_buffer.WriteRecord(std::move(vals));
 
-    if ( write_buffer_pos >= WRITER_BUFFER_SIZE || ! buf || run_state::terminating )
+    if ( write_buffer.Full() || ! buf || run_state::terminating )
         // Buffer full (or no buffering desired or terminating).
         FlushWriteBuffer();
 }
 
 void WriterFrontend::FlushWriteBuffer() {
-    if ( disabled ) {
-        CleanupWriteBuffer();
+    if ( disabled )
         return;
-    }
 
-    if ( ! write_buffer_pos )
+    if ( write_buffer.Empty() )
         // Nothing to do.
         return;
 
-    if ( backend )
-        backend->SendIn(new WriteMessage(backend, num_fields, write_buffer_pos, write_buffer));
+    auto records = std::move(write_buffer).TakeRecords();
 
-    // Clear buffer (no delete, we pass ownership to child thread.)
-    write_buffer = nullptr;
-    write_buffer_pos = 0;
+    // We've already pushed to broker during Write(). If another backend
+    // is used, push all the buffered log records to it now.
+    const bool broker_is_cluster_backend = zeek::cluster::backend == zeek::broker_mgr;
+    if ( remote && ! broker_is_cluster_backend )
+        zeek::cluster::backend->PublishLogWrites(header, Span{records});
+
+    if ( backend )
+        backend->SendIn(new WriteMessage(backend, num_fields, std::move(records)));
 }
 
 void WriterFrontend::SetBuf(bool enabled) {
@@ -263,24 +277,4 @@ void WriterFrontend::Rotate(const char* rotated_path, double open, double close,
         log_mgr->FinishedRotation(this, nullptr, nullptr, 0, 0, false, terminating);
 }
 
-void WriterFrontend::DeleteVals(int num_fields, Value** vals) {
-    // Note this code is duplicated in Manager::DeleteVals().
-    for ( int i = 0; i < num_fields; i++ )
-        delete vals[i];
-
-    delete[] vals;
-}
-
-void WriterFrontend::CleanupWriteBuffer() {
-    if ( ! write_buffer || write_buffer_pos == 0 )
-        return;
-
-    for ( int j = 0; j < write_buffer_pos; j++ )
-        DeleteVals(num_fields, write_buffer[j]);
-
-    delete[] write_buffer;
-    write_buffer = nullptr;
-    write_buffer_pos = 0;
-}
-
 } // namespace zeek::logging
