@@ -103,7 +103,7 @@ void HookArgument::Describe(ODesc* d) const {
 
         case FUNC:
             if ( arg.func )
-                d->Add(arg.func->Name());
+                d->Add(arg.func->GetName());
             else
                 d->Add("<null>");
             break;
@@ -249,6 +249,8 @@ void Plugin::InitPreScript() {}
 
 void Plugin::InitPostScript() {}
 
+void Plugin::InitPreExecution() {}
+
 Plugin::bif_item_list Plugin::BifItems() const { return bif_items; }
 
 void Plugin::Done() {
