@@ -30,8 +30,13 @@ void CPPCompile::DeclareLambda(const LambdaExpr* l, const ProfileFunc* pf) {
     auto l_id = l->Ingredients()->GetID();
     auto& ids = l->OuterIDs();
 
-    for ( auto id : ids )
-        lambda_names[id] = CaptureName(id);
+    for ( auto lid : ids ) {
+        if ( lambda_names.count(lid) > 0 ) {
+            ASSERT(lambda_names[lid] == CaptureName(lid));
+        }
+        else
+            lambda_names[lid] = CaptureName(lid);
+    }
 
     CreateFunction(l_id->GetType<FuncType>(), pf, lname, body, 0, l, FUNC_FLAVOR_FUNCTION);
 }
@@ -123,11 +128,18 @@ void CPPCompile::DeclareSubclass(const FuncTypePtr& ft, const ProfileFunc* pf, c
 
     if ( lambda_ids ) {
         for ( auto& id : *lambda_ids ) {
-            auto name = lambda_names[id];
+            const auto& name = lambda_names[id];
             auto tn = FullTypeName(id->GetType());
-            addl_args = addl_args + ", " + tn + " _" + name;
+            addl_args += ", ";
+            addl_args += tn;
+            addl_args += " _";
+            addl_args += name;
 
-            inits = inits + ", " + name + "(std::move(_" + name + "))";
+            inits += ", ";
+            inits += name;
+            inits += "(std::move(_";
+            inits += name;
+            inits += "))";
         }
     }
 
@@ -197,7 +209,7 @@ void CPPCompile::BuildLambda(const FuncTypePtr& ft, const ProfileFunc* pf, const
                              const LambdaExpr* l, const IDPList* lambda_ids) {
     // Declare the member variables for holding the captures.
     for ( auto& id : *lambda_ids ) {
-        auto name = lambda_names[id];
+        const auto& name = lambda_names[id];
         auto tn = FullTypeName(id->GetType());
         Emit("%s %s;", tn, name);
     }
