@@ -998,10 +998,23 @@ string CPPCompile::GenEQ(const Expr* e, GenType gt, const char* op, const char*
     auto tag = op1->GetType()->Tag();
     string negated(e->Tag() == EXPR_EQ ? "" : "! ");
 
-    if ( tag == TYPE_PATTERN )
-        return NativeToGT(negated + GenExpr(op1, GEN_DONT_CARE) + "->MatchExactly(" + GenExpr(op2, GEN_DONT_CARE) +
-                              "->AsString())",
-                          e->GetType(), gt);
+    if ( tag == TYPE_PATTERN ) {
+        auto gen1 = GenExpr(op1, GEN_DONT_CARE);
+        auto gen2 = GenExpr(op2, GEN_DONT_CARE);
+        string gen;
+
+        if ( op2->GetType()->Tag() == TYPE_PATTERN ) {
+            gen1 += "->AsPattern()->PatternText()";
+            gen2 += "->AsPattern()->PatternText()";
+
+            gen = "(strcmp(" + gen1 + ", " + gen2 + ") == 0)";
+        }
+
+        else
+            gen = gen1 + "->MatchExactly(" + gen2 + "->AsString())";
+
+        return NativeToGT(negated + gen, e->GetType(), gt);
+    }
 
     if ( tag == TYPE_FUNC ) {
         auto gen_f1 = GenExpr(op1, GEN_DONT_CARE);
@@ -1040,13 +1053,13 @@ string CPPCompile::GenDirectAssign(const ExprPtr& lhs, const string& rhs_native,
     if ( n->IsBlank() )
         return rhs_native;
 
-    auto name = IDNameStr(n);
+    const auto& name = IDNameStr(n);
 
     string gen;
 
     if ( n->IsGlobal() ) {
         const auto& t = n->GetType();
-        auto gn = globals[n->Name()];
+        const auto& gn = globals[n->Name()];
 
         if ( t->Tag() == TYPE_FUNC && t->AsFuncType()->Flavor() == FUNC_FLAVOR_EVENT ) {
             gen = string("set_event__CPP(") + gn + ", " + rhs_val_ptr + ", " + gn + "_ev)";
@@ -1156,8 +1169,12 @@ string CPPCompile::GenVectorOp(const Expr* e, string op1, string op2, const char
 
     auto invoke = string(vec_op) + "__CPP(" + op1 + ", " + op2 + ")";
 
-    if ( op2_t->Yield()->Tag() == TYPE_STRING )
+    auto tag2 = op2_t->Yield()->Tag();
+
+    if ( tag2 == TYPE_STRING )
         return string("str_vec_op_") + invoke;
+    if ( tag2 == TYPE_PATTERN )
+        return string("pat_vec_op_") + invoke;
 
     auto gen = string("vec_op_") + invoke;
 
