@@ -10,7 +10,7 @@ void CPPCompile::CompileFunc(const FuncInfo& func) {
     if ( ! IsCompilable(func) )
         return;
 
-    auto fname = Canonicalize(BodyName(func).c_str()) + "_zf";
+    auto fname = Canonicalize(BodyName(func)) + "_zf";
     auto pf = func.Profile();
     auto f = func.Func();
     const auto& body = func.Body();
@@ -19,7 +19,7 @@ void CPPCompile::CompileFunc(const FuncInfo& func) {
 }
 
 void CPPCompile::CompileLambda(const LambdaExpr* l, const ProfileFunc* pf) {
-    auto lname = Canonicalize(l->Name().c_str()) + "_lb";
+    auto lname = Canonicalize(l->Name()) + "_lb";
     auto body = l->Ingredients()->Body();
     auto l_id = l->Ingredients()->GetID();
     auto& ids = l->OuterIDs();
@@ -38,12 +38,18 @@ void CPPCompile::GenInvokeBody(const string& call, const TypePtr& t) {
 
 void CPPCompile::DefineBody(const FuncTypePtr& ft, const ProfileFunc* pf, const string& fname, const StmtPtr& body,
                             const IDPList* lambda_ids, FunctionFlavor flavor) {
+    IDPList l_ids;
+    if ( lambda_ids )
+        l_ids = *lambda_ids;
+
     locals.clear();
     params.clear();
 
     body_name = fname;
 
+    func_type = ft;
     ret_type = ft->Yield();
+
     in_hook = flavor == FUNC_FLAVOR_HOOK;
     auto ret_type_str = in_hook ? "bool" : FullTypeName(ret_type);
 
@@ -52,7 +58,7 @@ void CPPCompile::DefineBody(const FuncTypePtr& ft, const ProfileFunc* pf, const
 
     NL();
 
-    Emit("%s %s(%s)", ret_type_str, fname, ParamDecl(ft, lambda_ids, pf));
+    Emit("%s %s(%s)", ret_type_str, fname, ParamDecl(ft, &l_ids, pf));
 
     StartBlock();
 
@@ -64,7 +70,7 @@ void CPPCompile::DefineBody(const FuncTypePtr& ft, const ProfileFunc* pf, const
     InitializeEvents(pf);
 
     // Create the local variables.
-    DeclareLocals(pf, lambda_ids);
+    DeclareLocals(pf, &l_ids);
 
     GenStmt(body);
 
@@ -135,13 +141,15 @@ void CPPCompile::InitializeEvents(const ProfileFunc* pf) {
 }
 
 void CPPCompile::DeclareLocals(const ProfileFunc* pf, const IDPList* lambda_ids) {
-    // It's handy to have a set of the lambda captures rather than a list.
-    IDSet lambda_set;
+    // We track captures by their names rather than their ID*'s because the
+    // latter can be inconsistent when inlining.
+    set<string> capture_names;
     if ( lambda_ids )
         for ( auto li : *lambda_ids )
-            lambda_set.insert(li);
+            capture_names.insert(CaptureName(li));
 
     const auto& ls = pf->Locals();
+    int num_params = static_cast<int>(pf->Params().size());
 
     // Track whether we generated a declaration.  This is just for
     // tidiness in the output.
@@ -149,13 +157,13 @@ void CPPCompile::DeclareLocals(const ProfileFunc* pf, const IDPList* lambda_ids)
 
     for ( const auto& l : ls ) {
         auto ln = LocalName(l);
+        auto cn = CaptureName(l);
 
-        if ( lambda_set.count(l) > 0 )
-            // No need to declare these, they're passed in as
-            // parameters.
-            ln = lambda_names[l];
+        if ( capture_names.count(cn) > 0 )
+            // No need to declare these, they're passed in as parameters.
+            ln = cn;
 
-        else if ( params.count(l) == 0 ) { // Not a parameter, so must be a local.
+        else if ( params.count(l) == 0 && l->Offset() >= num_params ) { // Not a parameter, so must be a local.
             Emit("%s %s;", FullTypeName(l->GetType()), ln);
             did_decl = true;
         }
@@ -170,7 +178,7 @@ void CPPCompile::DeclareLocals(const ProfileFunc* pf, const IDPList* lambda_ids)
 string CPPCompile::BodyName(const FuncInfo& func) {
     const auto& f = func.Func();
     const auto& body = func.Body();
-    string fname = f->Name();
+    auto fname = f->GetName();
 
     // Extend name with location information.
     auto loc = body->GetLocationInfo();
