@@ -299,6 +299,11 @@ void FuncConstInfo::InitializerVals(std::vector<std::string>& ivs) const {
     }
 }
 
+void TypeConstInfo::InitializerVals(std::vector<std::string>& ivs) const {
+    auto& t = tv->GetType()->AsTypeType()->GetType();
+    ivs.emplace_back(Fmt(t->Tag()));
+}
+
 AttrInfo::AttrInfo(CPPCompile* _c, const AttrPtr& attr) : CompoundItemInfo(_c) {
     vals.emplace_back(Fmt(static_cast<int>(attr->Tag())));
     auto a_e = attr->GetExpr();
@@ -355,14 +360,16 @@ AttrsInfo::AttrsInfo(CPPCompile* _c, const AttributesPtr& _attrs) : CompoundItem
     }
 }
 
-GlobalLookupInitInfo::GlobalLookupInitInfo(CPPCompile* c, const ID* g, string _CPP_name)
+GlobalLookupInitInfo::GlobalLookupInitInfo(CPPCompile* c, const ID* g, string _CPP_name, bool do_init)
     : CPP_InitInfo(g), CPP_name(std::move(_CPP_name)) {
     Zeek_name = g->Name();
+    val = ValElem(c, do_init ? g->GetVal() : nullptr);
 }
 
 void GlobalLookupInitInfo::InitializerVals(std::vector<std::string>& ivs) const {
     ivs.push_back(CPP_name);
     ivs.push_back(string("\"") + Zeek_name + "\"");
+    ivs.push_back(val);
 }
 
 GlobalInitInfo::GlobalInitInfo(CPPCompile* c, const ID* g, string _CPP_name)
@@ -380,7 +387,8 @@ GlobalInitInfo::GlobalInitInfo(CPPCompile* c, const ID* g, string _CPP_name)
     else
         attrs = -1;
 
-    exported = g->IsExport();
+    is_exported = g->IsExport();
+    is_option = g->IsOption();
     val = ValElem(c, nullptr); // empty because we initialize dynamically
 
     if ( gt->Tag() == TYPE_FUNC && (! g->GetVal() || g->GetVal()->AsFunc()->GetKind() == Func::BUILTIN_FUNC) )
@@ -396,7 +404,8 @@ void GlobalInitInfo::InitializerVals(std::vector<std::string>& ivs) const {
     ivs.push_back(Fmt(type));
     ivs.push_back(Fmt(attrs));
     ivs.push_back(val);
-    ivs.push_back(Fmt(exported));
+    ivs.push_back(Fmt(is_exported));
+    ivs.push_back(Fmt(is_option));
     ivs.push_back(Fmt(func_with_no_val));
 }
 
@@ -540,7 +549,8 @@ void FuncTypeInfo::AddInitializerVals(std::vector<std::string>& ivs) const {
     ivs.emplace_back(Fmt(static_cast<int>(expressionless_return_okay)));
 }
 
-RecordTypeInfo::RecordTypeInfo(CPPCompile* _c, TypePtr _t) : AbstractTypeInfo(_c, std::move(_t)) {
+RecordTypeInfo::RecordTypeInfo(CPPCompile* _c, TypePtr _t, int _addl_fields)
+    : AbstractTypeInfo(_c, std::move(_t)), addl_fields(_addl_fields) {
     // Note, we leave init_cohort at 0 because the skeleton of this type
     // is built in the first cohort.
     auto r = t->AsRecordType()->Types();
@@ -570,6 +580,7 @@ RecordTypeInfo::RecordTypeInfo(CPPCompile* _c, TypePtr _t) : AbstractTypeInfo(_c
 
 void RecordTypeInfo::AddInitializerVals(std::vector<std::string>& ivs) const {
     ivs.emplace_back(Fmt(c->TrackString(t->GetName())));
+    ivs.emplace_back(Fmt(addl_fields));
 
     auto n = field_names.size();
 
