@@ -149,6 +149,14 @@ void CPP_IndexedInits<T>::Generate(InitsManager* im, std::vector<FuncValPtr>& iv
     ivec[offset] = lookup_func__CPP(fn, num_bodies, hashes, im->Types(t));
 }
 
+template<class T>
+void CPP_IndexedInits<T>::Generate(InitsManager* im, std::vector<TypeValPtr>& ivec, int offset,
+                                   ValElemVec& init_vals) const {
+    auto bt = base_type(static_cast<TypeTag>(init_vals[0]));
+    auto t = make_intrusive<TypeType>(bt);
+    ivec[offset] = make_intrusive<TypeVal>(t);
+}
+
 template<class T>
 void CPP_IndexedInits<T>::Generate(InitsManager* im, std::vector<AttrPtr>& ivec, int offset,
                                    ValElemVec& init_vals) const {
@@ -216,6 +224,7 @@ template class CPP_IndexedInits<RecordValPtr>;
 template class CPP_IndexedInits<TableValPtr>;
 template class CPP_IndexedInits<FileValPtr>;
 template class CPP_IndexedInits<FuncValPtr>;
+template class CPP_IndexedInits<TypeValPtr>;
 template class CPP_IndexedInits<AttrPtr>;
 template class CPP_IndexedInits<AttributesPtr>;
 template class CPP_IndexedInits<TypePtr>;
@@ -383,11 +392,18 @@ TypePtr CPP_TypeInits::BuildRecordType(InitsManager* im, ValElemVec& init_vals,
     auto r = cast_intrusive<RecordType>(inits_vec[offset]);
     ASSERT(r);
 
-    if ( r->NumFields() == 0 ) {
+    auto addl_fields = init_vals[2];
+
+    if ( addl_fields > 0 || r->NumFields() == 0 ) {
+        // We shouldn't be adding fields if the record doesn't have any
+        // existing fields - that would reflect an initialization botch.
+        if ( addl_fields > 0 && r->NumFields() == 0 )
+            reporter->InternalError("record unexpectedly empty when adding fields");
+
         type_decl_list tl;
 
         auto n = init_vals.size();
-        auto i = 2U;
+        auto i = 3U + addl_fields * 3;
 
         while ( i < n ) {
             auto s = im->Strings(init_vals[i++]);
@@ -457,11 +473,17 @@ int CPP_EnumMapping::ComputeOffset(InitsManager* im) const {
 
 void CPP_GlobalLookupInit::Generate(InitsManager* im, std::vector<void*>& /* inits_vec */, int /* offset */) const {
     global = find_global__CPP(name);
+    if ( val >= 0 )
+        // Have explicit initialization value.
+        global->SetVal(im->ConstVals(val));
 }
 
 void CPP_GlobalInit::Generate(InitsManager* im, std::vector<void*>& /* inits_vec */, int /* offset */) const {
     auto& t = im->Types(type);
-    global = lookup_global__CPP(name, t, exported);
+    global = lookup_global__CPP(name, t, is_exported);
+
+    if ( is_option )
+        global->SetOption();
 
     if ( ! global->HasVal() ) {
         if ( val >= 0 )
