@@ -6,7 +6,7 @@ gen_out=CPP-test/gen.$abbr
 
 (
     /bin/echo -n $1" "
-    if ! src/zeek -O gen-C++ --optimize-files=testing/btest $1 >&$gen_out 2>&1; then
+    if ! src/zeek -O gen-C++ --optimize-files=testing/btest $1 >$gen_out 2>&1; then
         echo "fail"
         exit 1
     fi
