@@ -74,6 +74,7 @@ static std::unordered_map<std::string, unsigned int> func_attrs = {
     {"Analyzer::__tag", ATTR_FOLDABLE},
     {"Cluster::Backend::ZeroMQ::spawn_zmq_proxy_thread", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"Cluster::Backend::__init", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Cluster::__listen_websocket", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"Cluster::__subscribe", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"Cluster::__unsubscribe", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"Cluster::make_event", ATTR_NO_SCRIPT_SIDE_EFFECTS},
@@ -112,6 +113,7 @@ static std::unordered_map<std::string, unsigned int> func_attrs = {
     {"Option::any_set_to_any_vec", ATTR_FOLDABLE},
     {"Option::set_change_handler", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"PacketAnalyzer::GTPV1::remove_gtpv1_connection", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"PacketAnalyzer::Geneve::get_options", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"PacketAnalyzer::TEREDO::remove_teredo_connection", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"PacketAnalyzer::__disable_analyzer", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"PacketAnalyzer::__enable_analyzer", ATTR_NO_SCRIPT_SIDE_EFFECTS},
@@ -146,6 +148,16 @@ static std::unordered_map<std::string, unsigned int> func_attrs = {
     {"Reporter::warning", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"Spicy::__resource_usage", ATTR_NO_ZEEK_SIDE_EFFECTS},
     {"Spicy::__toggle_analyzer", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Async::__close_backend", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Async::__erase", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Async::__get", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Async::__open_backend", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Async::__put", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Sync::__close_backend", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Sync::__erase", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Sync::__get", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Sync::__open_backend", ATTR_NO_SCRIPT_SIDE_EFFECTS},
+    {"Storage::Sync::__put", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"Supervisor::__create", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"Supervisor::__destroy", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"Supervisor::__is_supervised", ATTR_IDEMPOTENT},
@@ -181,6 +193,7 @@ static std::unordered_map<std::string, unsigned int> func_attrs = {
     {"any_set", ATTR_FOLDABLE},
     {"backtrace", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"bare_mode", ATTR_FOLDABLE},
+    {"blocking_lookup_hostname", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"bloomfilter_add", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"bloomfilter_basic_init", ATTR_NO_SCRIPT_SIDE_EFFECTS},
     {"bloomfilter_basic_init2", ATTR_NO_SCRIPT_SIDE_EFFECTS},
@@ -324,6 +337,7 @@ static std::unordered_map<std::string, unsigned int> func_attrs = {
     {"is_alnum", ATTR_FOLDABLE},
     {"is_alpha", ATTR_FOLDABLE},
     {"is_ascii", ATTR_FOLDABLE},
+    {"is_event_handled", ATTR_IDEMPOTENT}, // can error
     {"is_file_analyzer", ATTR_NO_ZEEK_SIDE_EFFECTS},
     {"is_icmp_port", ATTR_FOLDABLE},
     {"is_local_interface", ATTR_IDEMPOTENT},
@@ -339,6 +353,7 @@ static std::unordered_map<std::string, unsigned int> func_attrs = {
     {"is_v6_addr", ATTR_FOLDABLE},
     {"is_v6_subnet", ATTR_FOLDABLE},
     {"is_valid_ip", ATTR_FOLDABLE},
+    {"is_valid_subnet", ATTR_FOLDABLE},
     {"join_string_set", ATTR_FOLDABLE},
     {"join_string_vec", ATTR_FOLDABLE},
     {"levenshtein_distance", ATTR_FOLDABLE},
@@ -470,7 +485,7 @@ static std::unordered_map<std::string, unsigned int> func_attrs = {
     {"to_addr", ATTR_IDEMPOTENT},   // can error
     {"to_count", ATTR_IDEMPOTENT},  // can error
     {"to_double", ATTR_IDEMPOTENT}, // can error
-    {"to_int", ATTR_FOLDABLE},
+    {"to_int", ATTR_IDEMPOTENT},    // can error
     {"to_json", ATTR_FOLDABLE},
     {"to_lower", ATTR_FOLDABLE},
     {"to_port", ATTR_IDEMPOTENT}, // can error
