@@ -24,11 +24,12 @@ p_hash_type p_hash(const Obj* o) {
 
 ProfileFunc::ProfileFunc(const Func* func, const StmtPtr& body, bool _abs_rec_fields) {
     profiled_func = func;
+    profiled_scope = profiled_func->GetScope();
     profiled_body = body.get();
     abs_rec_fields = _abs_rec_fields;
 
-    auto ft = func->GetType()->AsFuncType();
-    auto& fcaps = ft->GetCaptures();
+    profiled_func_t = cast_intrusive<FuncType>(func->GetType());
+    auto& fcaps = profiled_func_t->GetCaptures();
 
     if ( fcaps ) {
         int offset = 0;
@@ -40,7 +41,20 @@ ProfileFunc::ProfileFunc(const Func* func, const StmtPtr& body, bool _abs_rec_fi
         }
     }
 
-    Profile(ft, body);
+    TrackType(profiled_func_t);
+    body->Traverse(this);
+
+    // Examine the locals and identify the parameters based on their offsets
+    // (being careful not to be fooled by captures that incidentally have low
+    // offsets). This approach allows us to accommodate function definitions
+    // that use different parameter names than appear in the original
+    // declaration.
+    num_params = profiled_func_t->Params()->NumFields();
+
+    for ( auto l : locals ) {
+        if ( captures.count(l) == 0 && l->Offset() < num_params )
+            params.insert(l);
+    }
 }
 
 ProfileFunc::ProfileFunc(const Stmt* s, bool _abs_rec_fields) {
@@ -56,6 +70,9 @@ ProfileFunc::ProfileFunc(const Expr* e, bool _abs_rec_fields) {
 
     if ( e->Tag() == EXPR_LAMBDA ) {
         auto func = e->AsLambdaExpr();
+        ASSERT(func->GetType()->Tag() == TYPE_FUNC);
+        profiled_scope = func->GetScope();
+        profiled_func_t = cast_intrusive<FuncType>(func->GetType());
 
         int offset = 0;
 
@@ -64,7 +81,17 @@ ProfileFunc::ProfileFunc(const Expr* e, bool _abs_rec_fields) {
             captures_offsets[oid] = offset++;
         }
 
-        Profile(func->GetType()->AsFuncType(), func->Ingredients()->Body());
+        auto ft = func->GetType()->AsFuncType();
+        auto& body = func->Ingredients()->Body();
+
+        num_params = ft->Params()->NumFields();
+
+        auto& ov = profiled_scope->OrderedVars();
+        for ( int i = 0; i < num_params; ++i )
+            params.insert(ov[i].get());
+
+        TrackType(ft);
+        body->Traverse(this);
     }
 
     else
@@ -73,14 +100,8 @@ ProfileFunc::ProfileFunc(const Expr* e, bool _abs_rec_fields) {
         e->Traverse(this);
 }
 
-void ProfileFunc::Profile(const FuncType* ft, const StmtPtr& body) {
-    num_params = ft->Params()->NumFields();
-    TrackType(ft);
-    body->Traverse(this);
-}
-
 TraversalCode ProfileFunc::PreStmt(const Stmt* s) {
-    stmts.push_back(s);
+    stmts.push_back({NewRef{}, const_cast<Stmt*>(s)});
 
     switch ( s->Tag() ) {
         case STMT_INIT:
@@ -164,7 +185,7 @@ TraversalCode ProfileFunc::PreStmt(const Stmt* s) {
 }
 
 TraversalCode ProfileFunc::PreExpr(const Expr* e) {
-    exprs.push_back(e);
+    exprs.push_back({NewRef{}, const_cast<Expr*>(e)});
 
     TrackType(e->GetType());
 
@@ -181,28 +202,10 @@ TraversalCode ProfileFunc::PreExpr(const Expr* e) {
             TrackType(id->GetType());
 
             if ( id->IsGlobal() ) {
-                globals.insert(id);
-                all_globals.insert(id);
-
-                const auto& t = id->GetType();
-                if ( t->Tag() == TYPE_FUNC )
-                    if ( t->AsFuncType()->Flavor() == FUNC_FLAVOR_EVENT )
-                        events.insert(id->Name());
-
+                PreID(id);
                 break;
             }
 
-            // This is a tad ugly.  Unfortunately due to the weird way
-            // that Zeek function *declarations* work, there's no reliable
-            // way to get the list of parameters for a function *definition*,
-            // since they can have different names than what's present in the
-            // declaration.  So we identify them directly, by knowing that
-            // they come at the beginning of the frame ... and being careful
-            // to avoid misconfusing a lambda capture with a low frame offset
-            // as a parameter.
-            if ( captures.count(id) == 0 && id->Offset() < num_params )
-                params.insert(id);
-
             locals.insert(id);
 
             break;
@@ -426,11 +429,6 @@ TraversalCode ProfileFunc::PreExpr(const Expr* e) {
             for ( const auto& i : l->OuterIDs() ) {
                 locals.insert(i);
                 TrackID(i);
-
-                // See above re EXPR_NAME regarding the following
-                // logic.
-                if ( captures.count(i) == 0 && i->Offset() < num_params )
-                    params.insert(i);
             }
 
             // In general, we don't want to recurse into the body.
@@ -487,10 +485,29 @@ TraversalCode ProfileFunc::PreExpr(const Expr* e) {
 TraversalCode ProfileFunc::PreID(const ID* id) {
     TrackID(id);
 
+    if ( id->IsGlobal() ) {
+        globals.insert(id);
+        all_globals.insert(id);
+
+        const auto& t = id->GetType();
+        TrackType(t);
+
+        if ( t->Tag() == TYPE_FUNC )
+            if ( t->AsFuncType()->Flavor() == FUNC_FLAVOR_EVENT )
+                events.insert(id->Name());
+    }
+
     // There's no need for any further analysis of this ID.
     return TC_ABORTSTMT;
 }
 
+TraversalCode ProfileFunc::PreType(const Type* t) {
+    TrackType(t);
+
+    // There's no need for any further analysis of this type.
+    return TC_ABORTSTMT;
+}
+
 void ProfileFunc::TrackType(const Type* t) {
     if ( ! t )
         return;
@@ -514,6 +531,12 @@ void ProfileFunc::TrackID(const ID* id) {
         // Already tracked.
         return;
 
+    if ( id->IsGlobal() ) {
+        globals.insert(id);
+        all_globals.insert(id);
+        TrackType(id->GetType());
+    }
+
     ordered_ids.push_back(id);
 }
 
@@ -546,7 +569,9 @@ void ProfileFunc::CheckRecordConstructor(TypePtr t) {
         }
 }
 
-ProfileFuncs::ProfileFuncs(std::vector<FuncInfo>& funcs, is_compilable_pred pred, bool _full_record_hashes) {
+ProfileFuncs::ProfileFuncs(std::vector<FuncInfo>& funcs, is_compilable_pred pred, bool _compute_func_hashes,
+                           bool _full_record_hashes) {
+    compute_func_hashes = _compute_func_hashes;
     full_record_hashes = _full_record_hashes;
 
     for ( auto& f : funcs ) {
@@ -554,10 +579,18 @@ ProfileFuncs::ProfileFuncs(std::vector<FuncInfo>& funcs, is_compilable_pred pred
 
         if ( ! pred || (*pred)(pf.get(), nullptr) )
             MergeInProfile(pf.get());
+        else if ( pred )
+            f.SetSkip(true);
 
         // Track the profile even if we're not compiling the function, since
         // the AST optimizer will still need it to reason about function-call
         // side effects.
+
+        // Propagate previous hash if requested.
+        auto prev_pf = f.Profile();
+        if ( ! compute_func_hashes && prev_pf && prev_pf->HasHashVal() )
+            pf->SetHashVal(prev_pf->HashVal());
+
         f.SetProfile(std::move(pf));
         func_profs[f.Func()] = f.ProfilePtr();
     }
@@ -637,7 +670,7 @@ bool ProfileFuncs::GetCallSideEffects(const NameExpr* n, IDSet& non_local_ids, T
 
     auto func = fv->AsFunc();
     if ( func->GetKind() == Func::BUILTIN_FUNC ) {
-        if ( has_script_side_effects(func->Name()) )
+        if ( has_script_side_effects(func->GetName()) )
             is_unknown = true;
         return true;
     }
@@ -805,15 +838,21 @@ void ProfileFuncs::ComputeTypeHashes(const std::vector<const Type*>& types) {
 }
 
 void ProfileFuncs::ComputeBodyHashes(std::vector<FuncInfo>& funcs) {
-    for ( auto& f : funcs )
-        if ( ! f.ShouldSkip() )
+    for ( auto& f : funcs ) {
+        if ( f.ShouldSkip() )
+            continue;
+        auto pf = f.ProfilePtr();
+        if ( compute_func_hashes || ! pf->HasHashVal() )
             ComputeProfileHash(f.ProfilePtr());
+    }
 
     for ( auto& l : lambdas ) {
         auto pf = ExprProf(l);
         func_profs[l->PrimaryFunc().get()] = pf;
         lambda_primaries[l->Name()] = l->PrimaryFunc().get();
-        ComputeProfileHash(pf);
+
+        if ( compute_func_hashes || ! pf->HasHashVal() )
+            ComputeProfileHash(pf);
     }
 }
 
@@ -824,12 +863,18 @@ void ProfileFuncs::ComputeProfileHash(std::shared_ptr<ProfileFunc> pf) {
     // prevent collisions due to elements with simple hashes
     // (such as Stmt's or Expr's that are only represented by
     // the hash of their tag).
+    h = merge_p_hashes(h, p_hash("params"));
+    auto& ov = pf->ProfiledScope()->OrderedVars();
+    int n = pf->NumParams();
+    for ( int i = 0; i < n; ++i )
+        h = merge_p_hashes(h, p_hash(ov[i]->Name()));
+
     h = merge_p_hashes(h, p_hash("stmts"));
-    for ( auto i : pf->Stmts() )
+    for ( auto& i : pf->Stmts() )
         h = merge_p_hashes(h, p_hash(i->Tag()));
 
     h = merge_p_hashes(h, p_hash("exprs"));
-    for ( auto i : pf->Exprs() )
+    for ( auto& i : pf->Exprs() )
         h = merge_p_hashes(h, p_hash(i->Tag()));
 
     h = merge_p_hashes(h, p_hash("ids"));
@@ -1370,8 +1415,8 @@ bool ProfileFuncs::AssessSideEffects(const SideEffectsOp* se, SideEffectsOp::Acc
 }
 
 std::shared_ptr<SideEffectsOp> ProfileFuncs::GetCallSideEffects(const ScriptFunc* sf) {
-    if ( lambda_primaries.count(sf->Name()) > 0 )
-        sf = lambda_primaries[sf->Name()];
+    if ( lambda_primaries.count(sf->GetName()) > 0 )
+        sf = lambda_primaries[sf->GetName()];
 
     auto sf_se = func_side_effects.find(sf);
     if ( sf_se != func_side_effects.end() )
@@ -1480,7 +1525,7 @@ ASTBlockAnalyzer::ASTBlockAnalyzer(std::vector<FuncInfo>& funcs) {
             continue;
 
         auto func = f.Func();
-        std::string fn = func->Name();
+        auto fn = func->GetName();
         auto body = f.Body();
 
         // First get the line numbers all sorted out.
