@@ -125,13 +125,13 @@ bool should_analyze(const ScriptFuncPtr& f, const StmtPtr& body) {
     return false;
 }
 
-bool obj_matches_opt_files(const Obj* obj) {
+bool filename_matches_opt_files(const char* filename) {
     auto& ofiles = analysis_options.only_files;
 
     if ( ofiles.empty() )
         return false;
 
-    auto fin = util::detail::normalize_path(obj->GetLocationInfo()->filename);
+    auto fin = util::detail::normalize_path(filename);
 
     for ( auto& o : ofiles )
         if ( std::regex_match(fin, o) )
@@ -140,6 +140,8 @@ bool obj_matches_opt_files(const Obj* obj) {
     return false;
 }
 
+bool obj_matches_opt_files(const Obj* obj) { return filename_matches_opt_files(obj->GetLocationInfo()->filename); }
+
 static bool optimize_AST(ScriptFuncPtr f, std::shared_ptr<ProfileFunc>& pf, std::shared_ptr<Reducer>& rc,
                          ScopePtr scope, StmtPtr& body) {
     pf = std::make_shared<ProfileFunc>(f.get(), body, true);
@@ -469,7 +471,7 @@ static void generate_CPP(std::shared_ptr<ProfileFuncs> pfs) {
     const bool standalone = analysis_options.gen_standalone_CPP;
     const bool report = analysis_options.report_uncompilable;
 
-    CPPCompile cpp(funcs, pfs, gen_name, standalone, report);
+    CPPCompile cpp(funcs, std::move(pfs), gen_name, standalone, report);
 }
 
 static void analyze_scripts_for_ZAM(std::shared_ptr<ProfileFuncs> pfs) {
@@ -517,7 +519,7 @@ static void analyze_scripts_for_ZAM(std::shared_ptr<ProfileFuncs> pfs) {
             if ( g->GetType()->Tag() != TYPE_FUNC )
                 continue;
 
-            auto v = g->GetVal();
+            const auto& v = g->GetVal();
             if ( v )
                 func_used_indirectly.insert(v->AsFunc());
         }
@@ -653,7 +655,7 @@ void analyze_scripts(bool no_unused_warnings) {
             reporter->FatalError("-O ZAM and -O gen-C++ conflict");
 
         auto pfs = std::make_shared<ProfileFuncs>(funcs, is_CPP_compilable, true, false);
-        generate_CPP(pfs);
+        generate_CPP(std::move(pfs));
         exit(0);
     }
 
