@@ -2,7 +2,6 @@
 
 #include "glue-compiler.h"
 
-#include <algorithm>
 #include <limits>
 #include <stdexcept>
 
@@ -422,7 +421,7 @@ void GlueCompiler::preprocessEvtFile(hilti::rt::filesystem::path& path, std::ist
             // Output empty line to keep line numbers the same
             out << '\n';
 
-            auto m = hilti::util::split1(trimmed);
+            auto m = hilti::util::split1(std::move(trimmed));
 
             if ( auto rc = pp.processLine(m.first, m.second); ! rc )
                 throw ParseError(rc.error());
@@ -480,19 +479,19 @@ bool GlueCompiler::loadEvtFile(hilti::rt::filesystem::path& path) {
             if ( looking_at(*chunk, 0, "protocol") ) {
                 auto a = parseProtocolAnalyzer(*chunk);
                 SPICY_DEBUG(hilti::util::fmt("  Got protocol analyzer definition for %s", a.name));
-                _protocol_analyzers.push_back(a);
+                _protocol_analyzers.push_back(std::move(a));
             }
 
             else if ( looking_at(*chunk, 0, "file") ) {
                 auto a = parseFileAnalyzer(*chunk);
                 SPICY_DEBUG(hilti::util::fmt("  Got file analyzer definition for %s", a.name));
-                _file_analyzers.push_back(a);
+                _file_analyzers.push_back(std::move(a));
             }
 
             else if ( looking_at(*chunk, 0, "packet") ) {
                 auto a = parsePacketAnalyzer(*chunk);
                 SPICY_DEBUG(hilti::util::fmt("  Got packet analyzer definition for %s", a.name));
-                _packet_analyzers.push_back(a);
+                _packet_analyzers.push_back(std::move(a));
             }
 
             else if ( looking_at(*chunk, 0, "on") ) {
@@ -517,7 +516,7 @@ bool GlueCompiler::loadEvtFile(hilti::rt::filesystem::path& path) {
                 else
                     SPICY_DEBUG(hilti::util::fmt("  Got module %s to import", module));
 
-                _imports.emplace_back(hilti::ID(module), std::move(scope));
+                _imports.emplace_back(hilti::ID(std::move(module)), std::move(scope));
             }
 
             else if ( looking_at(*chunk, 0, "export") ) {
@@ -563,7 +562,7 @@ bool GlueCompiler::loadEvtFile(hilti::rt::filesystem::path& path) {
     }
 
     for ( auto&& ev : new_events )
-        _events.push_back(ev);
+        _events.push_back(std::move(ev));
 
     return true;
 }
@@ -699,7 +698,7 @@ glue::ProtocolAnalyzer GlueCompiler::parseProtocolAnalyzer(const std::string& ch
 
                 case both:
                     a.unit_name_orig = unit;
-                    a.unit_name_resp = unit;
+                    a.unit_name_resp = std::move(unit);
                     break;
             }
         }
@@ -880,7 +879,7 @@ glue::Event GlueCompiler::parseEvent(const std::string& chunk) {
             eat_token(chunk, &i, ",");
 
         auto expr = extract_expr(chunk, &i);
-        ev.exprs.push_back(expr);
+        ev.exprs.push_back(std::move(expr));
         first = false;
     }
 
@@ -946,7 +945,7 @@ glue::Export GlueCompiler::parseExport(const std::string& chunk) {
 
             if ( looking_at(chunk, i, "&log") ) {
                 eat_token(chunk, &i, "&log");
-                export_.logs.insert(field);
+                export_.logs.insert(std::move(field));
             }
 
             if ( looking_at(chunk, i, "}") ) {
@@ -1194,7 +1193,7 @@ bool GlueCompiler::PopulateEvents() {
             acc.expression = e;
             acc.location = ev.location;
             // acc.dollar_id = util::startsWith(e, "$");
-            ev.expression_accessors.push_back(acc);
+            ev.expression_accessors.push_back(std::move(acc));
         }
     }
 
@@ -1205,33 +1204,12 @@ bool GlueCompiler::PopulateEvents() {
 
 #include <spicy/ast/visitor.h>
 
-// Helper visitor to wrap expressions using the TryMember operator into a
-// "deferred" expression.
-class WrapTryMemberVisitor : public spicy::visitor::MutatingPostOrder {
-public:
-    WrapTryMemberVisitor(Builder* builder, bool catch_exception)
-        : spicy::visitor::MutatingPostOrder(builder, logging::debug::GlueCompiler), _catch_exception(catch_exception) {}
-
-    void operator()(hilti::expression::UnresolvedOperator* n) final {
-        if ( n->kind() == hilti::operator_::Kind::TryMember )
-            replaceNode(n, builder()->expressionDeferred(n->as<hilti::Expression>(), _catch_exception), "wrap .?");
-    }
-
-private:
-    bool _catch_exception;
-};
-
 static hilti::Result<hilti::Expression*> parseArgument(Builder* builder, const std::string& expression,
                                                        bool catch_exception, const hilti::Meta& meta) {
     auto expr = ::spicy::builder::parseExpression(builder, expression, meta);
     if ( ! expr )
         return hilti::result::Error(hilti::util::fmt("error parsing event argument expression '%s'", expression));
 
-    // If the expression uses the ".?" operator, we need to defer evaluation
-    // so that we can handle potential exceptions at runtime.
-    auto v = WrapTryMemberVisitor(builder, catch_exception);
-    spicy::visitor::visit(v, *expr);
-
     return expr;
 }
 
@@ -1359,7 +1337,10 @@ bool GlueCompiler::CreateSpicyHook(glue::Event* ev) {
 
     body.addCall("zeek_rt::raise_event", {handler_expr, builder()->move(builder()->id("args"))}, meta);
 
-#if SPICY_VERSION_NUMBER >= 11200
+#if SPICY_VERSION_NUMBER >= 11300
+    auto attrs = builder()->attributeSet(
+        {builder()->attribute(hilti::attribute::kind::Priority, builder()->integer(ev->priority))});
+#elif SPICY_VERSION_NUMBER >= 11200
     auto attrs = builder()->attributeSet(
         {builder()->attribute(hilti::Attribute::Kind::Priority, builder()->integer(ev->priority))});
 #else
@@ -1367,7 +1348,7 @@ bool GlueCompiler::CreateSpicyHook(glue::Event* ev) {
 #endif
     auto parameters = hilti::util::transform(ev->parameters, [](const auto& p) { return p.get(); });
     auto unit_hook = builder()->declarationHook(parameters, body.block(), attrs, meta);
-    auto hook_decl = builder()->declarationUnitHook(ev->hook, unit_hook, meta);
+    auto hook_decl = builder()->declarationUnitHook(ev->hook, unit_hook, std::move(meta));
     ev->spicy_module->spicy_module->add(context(), hook_decl);
 
     return true;
@@ -1536,7 +1517,7 @@ struct VisitorZeekType : spicy::visitor::PreOrder {
         });
 
         auto tmp = builder->addTmp(tmpName("labels", id()),
-                                   builder->typeVector(
+                                   builder->typeSet(
                                        builder->qualifiedType(builder->typeTuple(hilti::QualifiedTypes{
                                                                   builder->qualifiedType(builder->typeString(),
                                                                                          hilti::Constness::Const),
@@ -1545,9 +1526,8 @@ struct VisitorZeekType : spicy::visitor::PreOrder {
                                                               hilti::Constness::Const)));
 
         for ( const auto& l : t->labels() )
-            builder->addMemberCall(tmp, "push_back",
-                                   {builder->tuple(
-                                       {builder->stringLiteral(l->id().str()), builder->integer(l->value())})});
+            builder->addExpression(builder->add(tmp, builder->tuple({builder->stringMutable(l->id().str()),
+                                                                     builder->integer(l->value())})));
 
         result(builder->call("zeek_rt::create_enum_type", {builder->stringMutable(id().namespace_().str()),
                                                            builder->stringMutable(id().local().str()), tmp}));
