@@ -16,14 +16,12 @@
 
 #include <libgen.h>
 #include <unistd.h>
-#include <array>
 #include <cinttypes>
 #include <cstdarg>
 #include <cstdint>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
-#include <memory> // std::unique_ptr
 #include <string>
 #include <string_view>
 #include <vector>
@@ -97,6 +95,17 @@ namespace filesystem = ghc::filesystem;
 inline constexpr std::string_view path_list_separator = ":";
 #endif
 
+#include "zeek/3rdparty/nonstd/expected.hpp"
+namespace zeek {
+template<typename T, typename E>
+using expected = nonstd::expected<T, E>;
+
+template<typename E>
+using unexpected = nonstd::unexpected<E>;
+} // namespace zeek
+
+#include "zeek/Span.h"
+
 using zeek_int_t = int64_t;
 using zeek_uint_t = uint64_t;
 
@@ -112,6 +121,10 @@ namespace zeek {
 class ODesc;
 class RecordVal;
 
+// Byte buffer types used by serialization code in storage and cluster.
+using byte_buffer = std::vector<std::byte>;
+using byte_buffer_span = Span<const std::byte>;
+
 namespace util {
 namespace detail {
 
@@ -590,11 +603,11 @@ template<typename T>
 std::vector<T> split(T s, const T& delim) {
     // If there's no delimiter, return a copy of the existing string.
     if ( delim.empty() )
-        return {T(s)};
+        return {std::move(s)};
 
     // If the delimiter won't fit in the string, just return a copy as well.
     if ( s.size() < delim.size() )
-        return {T(s)};
+        return {std::move(s)};
 
     std::vector<T> l;
 
