@@ -354,7 +354,7 @@ void Manager::Redef(const zeek::detail::ID* id, const string& path, zeek::detail
         return;
     }
 
-    id_info->AddRedef(from_script, ic, std::move(init_expr), comment_buffer);
+    id_info->AddRedef(from_script, ic, std::move(init_expr), std::move(comment_buffer));
     script_info->AddRedef(id_info);
     comment_buffer.clear();
     last_identifier_seen = id_info;
